import java.util.Vector;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

// this package contains all the robot-classes
// like Motor or RobotInterface
import de.jaetzold.art.Motor;
import de.jaetzold.art.StepperMotor;
import de.jaetzold.art.Servo;
import de.jaetzold.art.Actuator;
import de.jaetzold.art.RawSensor;
import de.jaetzold.art.StateSensor;
import de.jaetzold.art.CountSensor;
import de.jaetzold.art.AngleSensor;
import de.jaetzold.art.BooleanSensor;
import de.jaetzold.art.LightSensor;
import de.jaetzold.art.Sensor;
import de.jaetzold.art.SensorEvent;
import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.RobotInterfaceFactory;
import de.jaetzold.art.RobotInterfaceDefinition;
import de.jaetzold.art.RobotInterfaceStringDefinition;
import de.jaetzold.art.ActuatorPort;
import de.jaetzold.art.SensorPort;
import de.jaetzold.art.examples.RCXRotationSensorStateDecider;
import de.jaetzold.art.examples.RCXRotationCountStateDecider;
import de.jaetzold.art.examples.RCXRotationFastCountStateDecider;
import de.jaetzold.art.examples.RCXRotationFasterCountStateDecider;
import de.jaetzold.art.examples.FTRotationCountStateDecider;
import de.jaetzold.util.StateDecider;
import de.jaetzold.util.ConversionAdapter;

public class MainTester {
	public static void main(String[] argv) {
		// first search for interfaces
		RobotInterface iface;
		if(argv.length>0) {
			// any argument on the commandline is interpreted as a portidentifier
			// look only for interfaces on ports which are given on
			// the command-line like "CMCOM1" or "CMCOM2"
			Vector v = new Vector();
			for(int i=0; i<argv.length; i++) {
				v.add(new RobotInterfaceStringDefinition(argv[i]));
			}
			RobotInterfaceDefinition[] iports =
					new RobotInterfaceDefinition[v.size()];
			v.copyInto(iports);
			iface = new RobotInterfaceFactory().getInterface(iports);
		} else {
			// let the class look for itself at all known ports
			iface = new RobotInterfaceFactory().getInterface();
		}
		if(iface == null) {
			System.err.println("No Interface could be found ...");
			System.exit(1);
		}
		
		final Motor countActuator = new Motor();
		final StepperMotor angleServo = new StepperMotor();
		// Look for available ActuatorPorts
		ActuatorPort[] aPorts = iface.getActuatorPorts();
		Panel aPanel = new Panel();
		aPanel.setLayout(new GridLayout(aPorts.length, 1));
		for(int i=0; i<aPorts.length; i++) {
			// create a new Motor for every ActuatorPort
			Motor m = new Motor();
			if(i==0) {
				//m = countActuator;
				m = angleServo;
			}
			// just assume that it is capable of connecting to a motor
			aPorts[i].connectWith(m);			
			// by now the motor can be used and every
			// command send to the Motor-Object will be reflected by the actual
			// motor from the robot (if there is any)

			// create AWTView to play with the motor:
			AWTActuatorView av = new AWTMotorView(m);
			aPanel.add(av);
		}	
		
		final CountSensor actuatorCountSensor = new CountSensor();
		final AngleSensor servoSensor = new AngleSensor(8);
		
		final StateSensor rcxRotationStateSensor = new StateSensor();
		rcxRotationStateSensor.setStateDecider(
								new RCXRotationSensorStateDecider());
		
		final CountSensor rcxRotationCountSensor = new CountSensor();
		rcxRotationCountSensor.setStateDecider(
								new RCXRotationSensorStateDecider());
		rcxRotationCountSensor.setCountDecider(
								new RCXRotationCountStateDecider());
		
		final CountSensor rcxRotationFastCountSensor = new CountSensor();
		rcxRotationFastCountSensor.setStateDecider(
								new RCXRotationSensorStateDecider());
		rcxRotationFastCountSensor.setCountDecider(
								new RCXRotationFastCountStateDecider());
		
		final CountSensor rcxRotationFasterCountSensor = new CountSensor();
		rcxRotationFasterCountSensor.setStateDecider(
								new RCXRotationSensorStateDecider());
		rcxRotationFasterCountSensor.setCountDecider(
								new RCXRotationFasterCountStateDecider());
		rcxRotationFasterCountSensor.connectWith(
								rcxRotationStateSensor.getSensorPort());
				
	//	StateDecider ftRotationSensorStateDecider = new MultiplicationStateDecider(new SensorStateDecider(countActuator), new CountSensor.AbsoluteStateDifferenceStateDecider());
		StateDecider ftRotationSensorStateDecider =
						new FTRotationCountStateDecider(angleServo);
		actuatorCountSensor.setCountDecider(ftRotationSensorStateDecider);
		servoSensor.setCountDecider(ftRotationSensorStateDecider);
		
		angleServo.setPrecision(1);
		angleServo.setCountSensor(actuatorCountSensor);
		
		// Look for available SensorPorts
		SensorPort[] sPorts = iface.getSensorPorts();
		Panel sPanel = new Panel();
		sPanel.setLayout(new GridLayout(sPorts.length, 1));
		for(int i=0; i<sPorts.length; i++) {
			// create a new Sensor for every SensorPort
			Sensor rs = new RawSensor();
			Sensor ss = new StateSensor();
			Sensor cs = new CountSensor();
			Sensor as = new AngleSensor(8);
			Sensor bs = new BooleanSensor();
			LightSensor ls = new LightSensor();
			ls.setScalingConversion(
				new ConversionAdapter() {
					public double convert(double value) {
						return value;
					}
				}
			);
			ls.setMeasurableMin(sPorts[i].getMinimumValue());
			ls.setMeasurableMax(sPorts[i].getMaximumValue());			
			
			if(i==0) {
				cs = actuatorCountSensor;
				as = servoSensor;
			}
			if(i==2) {
				ss = rcxRotationStateSensor;
			}
			if(i==2) {
				cs = rcxRotationFastCountSensor;
			}
			// and connect it with the corresponding Port
			sPorts[i].connectWith(ls);			
			sPorts[i].connectWith(rs);			
			sPorts[i].connectWith(ss);
			if(i!=2) {		
				sPorts[i].connectWith(cs);
			}			
			sPorts[i].connectWith(as);			
			sPorts[i].connectWith(bs);			
			// by now the sensor can be used

			// create AWTView to watch the sensor:
/*			AWTSensorView rsv = new AWTSensorView(rs);
			AWTSensorView ssv = new AWTSensorView(ss);
			AWTSensorView csv = new AWTSensorView(cs);
			AWTSensorView asv = new AWTSensorView(as);
			AWTSensorView bsv = new AWTBooleanSensorView(bs);
			AWTSensorView lsv = new AWTSensorView(ls);
			
			Panel spi = new Panel();
			spi.add(rsv);
			spi.add(ssv);
			spi.add(csv);
			spi.add(asv);
			spi.add(bsv);
			spi.add(lsv);
			
			sPanel.add(spi);*/
		}
		
		// create View for the sensor- and actuator-Panels:
		Frame f = new Frame();
		f.setLayout(new BorderLayout());
		f.add(aPanel, "Center");
		f.add(sPanel, "West");

		f.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent we) {
					System.exit(0);
				}
		});

		f.pack();
		f.show();
	}
}

class AWTBooleanSensorView	extends	AWTSensorView {
	TextField booleanOutput = new TextField(5);
	public AWTBooleanSensorView() {
	}
	public AWTBooleanSensorView(Sensor s) {
		super(s);
		if(s instanceof BooleanSensor) {
			booleanOutput.setText(((BooleanSensor)s).getBooleanValue() +"");
		}
	}
	{
		// create new root-contentPanel
		Panel contentPanel = new Panel();
		contentPanel.setLayout(new BorderLayout());
		// remove the contentPanel of the superclass
		remove(this.contentPanel);
		// and add it to our new root-panel
		contentPanel.add("West", this.contentPanel);
		// then change the classes reference to point to the new Panel
		this.contentPanel = contentPanel;
		// and add that Panel as the only one to this
		add(contentPanel);
		
		contentPanel = new Panel();
		this.contentPanel.add("Center", contentPanel);
		
		contentPanel.add(new Label(" as boolean:"));
		contentPanel.add(booleanOutput);
		booleanOutput.setEditable(false);
	}
	public void propertyChange(PropertyChangeEvent pce) {
		super.propertyChange(pce);
		if(pce.getPropertyName().equals("value")
				&& pce.getSource() instanceof BooleanSensor) {
			booleanOutput.setText(
					((BooleanSensor)pce.getSource())
						.convertToBoolean(
								((SensorEvent)pce.getNewValue())
									.getValue()) +"");
		}
	}
}

class AWTSensorView		extends		Panel
						implements	PropertyChangeListener
{
	TextField valueOutput = new TextField(5);
	protected Panel contentPanel = new Panel();
	
	public AWTSensorView() {
	}
	public AWTSensorView(Sensor s) {
		s.addPropertyChangeListener(this);
		valueOutput.setText(s.getValue() +"");
	}
	
	{
		valueOutput.setEditable(false);

		contentPanel.add(new Label(" sensor value:"));
		contentPanel.add(valueOutput);
		add(contentPanel);
	}
	
	public void propertyChange(PropertyChangeEvent pce) {
		if(pce.getPropertyName().equals("value")
				&& pce.getSource() instanceof Sensor) {
			valueOutput.setText(pce.getNewValue().toString());
		}
	}
}

class AWTActuatorView	extends		Panel
						implements	PropertyChangeListener
{
	public AWTActuatorView() {
	}
	public AWTActuatorView(final Actuator actionTarget) {

		actionTarget.addPropertyChangeListener(this);

		addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					double value = 0;
					try {
						value = Double.parseDouble(ae.getActionCommand());
					} catch(NumberFormatException nfe) {
						return;
					}
					actionTarget.setValue(value);
				}
		});
		valueOutput.setText(actionTarget.getValue() +"");
	}
	
	TextField valueOutput = new TextField(5);
	TextField valueInput = new TextField(5);
	protected Panel contentPanel = new Panel();
	{
		valueOutput.setEditable(false);
		
		contentPanel.add(new Label(" value set:"));
		contentPanel.add(valueInput);
		contentPanel.add(new Label(" value get:"));
		contentPanel.add(valueOutput);
		add(contentPanel);
	}
	public void propertyChange(PropertyChangeEvent pce) {
		String propName = pce.getPropertyName();
		if(propName.equals("value")) {
			if(pce.getSource() instanceof Actuator) {
				valueOutput.setText(pce.getNewValue().toString());
			}
		}
	}
	public void addActionListener(ActionListener al) {
		valueInput.addActionListener(al);
	}
	public void removeActionListener(ActionListener al) {
		valueInput.removeActionListener(al);
	}
}

class AWTMotorView extends AWTActuatorView {
	Motor motor;
	public AWTMotorView(Motor motor) {
		super(motor);
		this.motor = motor;
		power.setText(motor.getPower() +"");
	}
	Button on = new Button("on");
	Button off = new Button("off");
	Button forward = new Button("forward");
	Button backward = new Button("backward");
	Button reverse = new Button("reverse");
	TextField power = new TextField(5);
	{
		Panel contentPanel = new Panel();
		contentPanel.setLayout(new BorderLayout());
		remove(this.contentPanel);
		contentPanel.add("North", this.contentPanel);
		this.contentPanel = contentPanel;
		add(contentPanel);
		contentPanel = new Panel();
		this.contentPanel.add("Center", contentPanel);
		
		contentPanel.add(on);
		contentPanel.add(off);
		contentPanel.add(forward);
		contentPanel.add(backward);
		contentPanel.add(reverse);
		contentPanel.add(new Label(" power:"));
		contentPanel.add(power);
		on.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					motor.on();
				}
		});
		off.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					motor.off();
				}
		});
		forward.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					motor.forward();
				}
		});
		backward.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					motor.backward();
				}
		});
		reverse.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					motor.reverse();
				}
		});
		power.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					double p = 0;
					try {
						p = Double.parseDouble(ae.getActionCommand());
					} catch(NumberFormatException nfe) {
						return;
					}
					motor.setPower(p);
				}
		});
	}
	public void propertyChange(PropertyChangeEvent pce) {
		super.propertyChange(pce);
		if(pce.getSource() instanceof Actuator) {
			String propName = pce.getPropertyName();
			// by now this won't happen if the power from the pce is different
			// that's because we don't have a MotorPeer
			// and Motor doesn't know that by now
			if(propName.equals("power")) {
				power.setText(pce.getNewValue().toString());
			}
		}
	}
}
