package de.jaetzold.art;

import de.jaetzold.art.platform.ActuatorPeer;

/** Interface which describes the capabilities of an ActuatorPort. It guarantees that all SensorPeers delivered are actually instances of ActuatorPeer.
	*/
public interface ActuatorPort extends SensorPort {
	/**	Returns an ActuatorPeer which can be used by the given Actuator.
		There should never be the need in calling this method from somewhere outside of an Actuator-implementation.
		@see #connectWith(Actuator)
		@see RawActuator#connectWith(ActuatorPort)
		*/
	public ActuatorPeer getPeerFor(Actuator actuator);
	/**	Connects the given Actuator to this port.
		This sets the peer for the Actuator and disconnects it from any other
		port it was previously connected to.
		*/
	public void connectWith(Actuator actuator);
	/**	Disconnects the given Actuator from this port. If this has been the
		last Actuator connected with this port it will automatically become
		disabled, that means all associated resources are freed.
		*/
	public void disconnect(Actuator actuator);
	/**	Returns the type of this port as a String. Typical types could be "DC" or "Variable". This feature is not fully used and for now the String should be in some human-understandable form.
		*/
	public String getType();
}
