package de.jaetzold.art;

import de.jaetzold.art.platform.BooleanSensorPeer;

import de.jaetzold.util.BooleanConversion;

// TBD: this one is also a candidate to be implemented by a delegate/StateDecider
// new BooleanSensorEvent?
// filter Events for changes in the BooleanValue?
// generate new Event when the delegate changes?

// currently obsolete: or by bean-style listening to the property <code>booleanValue</code>.
/**	A class which represents a sensor for two states, false and true.
	The state can be retrieved through a call to <code>getBooleanValue()</code> or by converting the normal sensor value (which comes e.g. with a <code>SensorEvent</code>) with the method <code>convertToBoolean(double)</code>.
	*/
public class BooleanSensor extends StateSensor
{	
	/**	Returns the state of this Sensor as a boolean.
		@see #convertToBoolean(double)
		*/
	public boolean getBooleanValue() {
		return convertToBoolean(getValue());
	}
	
	/**	This method is used to convert the double values to boolean. Because it has <code>public</code>-access it normal double-values can be used to represent the state of this sensor. They are only converted to boolean if needed. For example as it is the case for interpreting the value of a <code>SensorEvent</code> which does not cover the boolean-type.
		*/
	public boolean convertToBoolean(double value) {
		boolean result = value!=0;
		
		BooleanConversion localConversion = booleanConversionDelegate;
		if(localConversion != null) {
			result = localConversion.convertToBoolean(value);
		}
		
		if(alternateReality) {
			result = !result;
		}
		return result;
	}
	
	/**	Returns a <code>BooleanConversion</code> which converts doubles to boolean in the exact same way as this <code>BooleanSensor</code> does. The type of conversion even changes when the one of this <code>BooleanSensor</code> changes because its <code>convertToBoolean(double)</code>-method is used for the actual conversion.
		@see #convertToBoolean(double)
		*/
	public BooleanConversion getBooleanConversion() {
		return new BooleanConversion() {
			// beware, since the delegate can change, this
			// is against the contract of Conversion!
			public boolean isCacheable() {
				boolean result = true;

				BooleanConversion localConversion = booleanConversionDelegate;
				if(localConversion != null) {
					result = localConversion.isCacheable();
				}
				return result;
			}
			public boolean convertToBoolean(double value) {
				return BooleanSensor.this.convertToBoolean(value);
			}
		};
	}

	private BooleanConversion booleanConversionDelegate;
	/**	This delegate will be used for the conversion of doubles to boolean.
		The boolean value returned from the delegate might be inverted afterwards when the property <code>alternateReality</code> is set to true.
		@see #convertToBoolean(double)
		@see #setAlternateReality(boolean)
		*/
	public void setBooleanConversionDelegate(BooleanConversion delegate) {
		this.booleanConversionDelegate = delegate;
	}
	/**	The delegate used for the conversion of doubles to boolean.
		@see #setBooleanConversionDelegate(BooleanConversion)
		*/
	public BooleanConversion getBooleanConversionDelegate() {
		return booleanConversionDelegate;
	}
	
	// couldn't resist to call the property like that :-)
	private boolean alternateReality;
	/**	Tells whether the acutally computed boolean will be inverted before delivering it to "the outside".
		@see #setAlternateReality(boolean)
		*/
	public boolean isAlternateReality() {
		return alternateReality;
	}
	/**	Confgures whether the acutally computed boolean will be inverted before delivering it to "the outside".
		This is kind of a "reversed"-property for <code>BooleanSensor</code>s.
		@see RawSensor#setReversed(boolean)
		*/
	public void setAlternateReality(boolean alternateReality) {
		this.alternateReality = alternateReality;
	}
}
