package de.jaetzold.art;

import de.jaetzold.art.platform.BaseSensorPort;
import de.jaetzold.art.platform.SensorPeer;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import de.jaetzold.util.NotConnectedException;
import de.jaetzold.util.Conversion;
import de.jaetzold.util.Debug;

// change values from long/double to int/double if double can have all ints
// done. But have to mention why not (4*conversion-methods and big nested if-statements), but why i tried first (greater precision for big values)

// change the conversion-delegate/statedecider/countmatrix-stuff
// to be more homogen
// maybe just one function for all?

// A Sensor is no Conversion, it's more of a value
// but a sensor converts something nevertheless, therefore it is maybe
// a good idea to be able to get a conversion from it which resembles
// the conversion it does on values
// should that be constant (the conversion at the time the sensor has been asked)
// or mutable (the conversion which converts always the same way as the sensor it is coming from)
// tend to constant or choice between them
// if choice maybe like: the sensor itself is the mutable conversion
// and the sensor can deliver a constant(probably cacheable) conversion

// conversionDelegates wegen RawPort ueberfluessig? Sollten eigentlich sowieso
// nur gesetzt werden wenn der Sensor erzeugt wird, da ansonsten welche
// geloescht werden koennten die gebraucht werden. vielleicht besser
// "addDelegate?"

/*	All events from the Peer come through the PropertyChange-Channel.
	All SensorListeners have an adapter in changes
	There comes every Change. If that is too much communication the SensorEventDescriptor-Feature should be implemented completely. Either by using a different Event-Channel, or by mapping descriptors to Strings. */
	// only Peer-side events by now.

/**	This class represents a basic input-device of a hardware-interface.
	Each device, such as a Sensor needs to be connected to
	a Port first. Through connecting with a Port this Object gets its
	peer which is required for proper use.
	@see #connectWith(Port)
	*/
public class RawSensor implements Sensor, SensorPeer {
	private Debug debug = new Debug("de.jaetzold.art.RawSensor");
	{ if(debug.debug) debug.objectIdentifier = debug.getIdentifierFor(this); }
	// the base raw sensor doesn't convert the value by itself, but can
	// have a delegate who does (hook2)
	// or a subclass which overrides this method (hook1)
	// (but should still call super.convertXX() to still let the delegate do its conversion or not? at least doesn't have to)
	// contract for delegates is, that
	// the incoming delegate comes first (first call super in subclasses!)
	// then the subclass does its incoming conversion
	// then the subclass does its outgoing conversion
	// and then comes the outgoing delegate
	
	// do we really need an incoming delegate?
	// no, but does it hurt?
	
	// convertToOutgoing not necessary because getValue() already does the trick

	/**	Contructs a new RawSensor.
		It still needs a peer. Through connecting with a Port this Object gets its
		peer which is required for proper use.
		@see #connectWith(Port)
		*/
	public RawSensor() {
	}

	double reverseConversion(double value) {
		// convertToIncoming and convertToActuatorOutgoing
		// both need this conversion, that's why it is in a seperate method
		return reversed ? value*-1 : value;
	}
	
	/** This is a Template-Method for subclasses.
		It converts the value as it comes from the peer into the internal value of this sensor. It is called whenever a new value comes from the peer. Because this Method also calls the incomingConversionDelegate (if one exists) it should not be overridden without calling <code>super</code>.
		@see #convertToOutgoing(double)
		@see #isIncomingConversionCacheable()
		*/
	protected double convertToIncoming(double peerValue) {
		double result = reverseConversion(peerValue);
		if(debug.debug) {
			debug.printInfo("convertToIncoming(" +peerValue +")"
						+(reversed ? ", reversed: " +(result) : ""));
		}
		Conversion localConversion = incomingConversionDelegate;
		if(localConversion != null) {
			result = localConversion.convert(peerValue);
		}
		return result;
	}
	
	/** This is a Template-Method for subclasses.
		It converts the internal value into the value which this sensor represents to the outside. It is called when the Sensor is asked for its value or sends a SensorEvent. Because this Method also calls the outgoingConversionDelegate (if one exists) it should not be overridden without calling <code>super</code>.
		@see #convertToIncoming(double)
		@see #isOutgoingConversionCacheable()
		*/
	protected double convertToOutgoing(double internalValue) {
		double result = internalValue;
		Conversion localConversion = outgoingConversionDelegate;
		if(localConversion != null) {
			result = localConversion.convert(internalValue);
		}
		return result;
	}
	
	/**	This is false by default.
		@see #setReversed(boolean)
		*/
	private boolean reversed = false;
	/**	Tells whether this Sensor treats negative values as positive and the other way round.
		@see #setReversed(boolean)
		*/
	public boolean isReversed() {
		return reversed;
	}
	/**	Controls whether this Sensor should treat negative values as positive and the other way round. That way this state defines what "+" in the hardware means for the algorithm.
		*/
	public void setReversed(boolean reversed) {
		boolean oldReversed = this.reversed;
		this.reversed = reversed;
		if(reversed != oldReversed) {
			update();
		}
	}
	
	/**	This is false by default.
		@see #setPrecision(double)
		*/
	protected boolean precisionSet = false;
	/**	The local value for precision. This is only valid if precisionSet is true.
		@see #precisionSet
		@see #setPrecision(double)
		*/
	protected double precision;
	public double getPrecision() {
		if(precisionSet) {
			return precision;
		} else {
			return peer.getPrecision();
		}
	}
	/**	Sets the local value for precision. This is not required because usually the precision is taken from the peer. However it may be useful to tune the value for precision according to the algorithm which will use this Sensor.
		@see #precisionSet
		@see #setPrecision(double)
		*/
	public void setPrecision(double precision) {
		this.precision = precision;
		precisionSet = true;
	}
		
	/** This is <em>not</em> the same as <code>isCacheable()</code> from Conversion or StateDecider. The Conversion may depend on a delegate and that delegate can be set more than once. Therefore it would be impossible to assure that the Conversion remains cacheable. But the contract is, that as long as the delegate doesn't change you can assume that the incoming conversion remains cacheable once it has become so.
		@see #isOutgoingConversionCacheable()
		@see #setIncomingConversionDelegate(Conversion)
		@see Conversion#isCacheable()
	*/
	protected boolean isIncomingConversionCacheable() {
		Conversion localConversion = incomingConversionDelegate;
		if(localConversion != null) {
			localConversion.isCacheable();
		}
		return true;
	}
	/** Same meaning as <code>isIncomingConversionCacheable()</code> but for the outgoingConversion.
		@see #isIncomingConversionCacheable()
		*/
	protected boolean isOutgoingConversionCacheable() {
		Conversion localConversion = outgoingConversionDelegate;
		if(localConversion != null) {
			localConversion.isCacheable();
		}
		return true;
	}
	
	private Conversion incomingConversionDelegate;
	/**	This is the delegate for converting peer-side values to the internal representation of this Sensor.
		Because there can be only one delegate be careful not to accidentally delete one that is already existent. If a conversion is necessary it is usually also possible to use another Sensor as a decorator which is connected to the SensorPort delivered by <code>getSensorPort()</code>.
		@see #convertToIncoming(double)
		@see #setOutgoingConversionDelegate(Conversion)
		@see #getSensorPort()
		*/
	public void setIncomingConversionDelegate(Conversion delegate) {
		synchronized(cacheMonitor) {
			if(delegate == null
					|| !delegate.equals(incomingConversionDelegate)) {
				incomingDoubleCacheValid = false;
			}
			this.incomingConversionDelegate = delegate;
		}
	}
	/**	Returns the delegate for converting peer-side values to the internal representation of this Sensor.
		@see #convertToIncoming(double)
		@see #setIncomingConversionDelegate(Conversion)
		*/
	public Conversion getIncomingConversionDelegate() {
		return incomingConversionDelegate;
	}
	private Conversion outgoingConversionDelegate;
	/**	This is the delegate for converting the internal value-representation of this Sensor to the one that is presented to the outside.
		Because there can be only one delegate be careful not to accidentally delete one that is already existent. If a conversion is necessary it is usually also possible to use another Sensor as a decorator which is connected to the SensorPort delivered by <code>getSensorPort()</code>.
		@see #convertToOutgoing(double)
		@see #setIncomingConversionDelegate(Conversion)
		@see #getSensorPort()
		*/
	public void setOutgoingConversionDelegate(Conversion delegate) {
		synchronized(cacheMonitor) {
			if(delegate == null
					|| !delegate.equals(outgoingConversionDelegate)) {
				outgoingDoubleCacheValid = false;
			}
			this.outgoingConversionDelegate = delegate;
		}
	}
	/**	Returns the delegate for converting the internal value-representation of this Sensor to the one that is presented to the outside.
		@see #convertToOutgoing(double)
		@see #setOutgoingConversionDelegate(Conversion)
		*/
	public Conversion getOutgoingConversionDelegate() {
		return outgoingConversionDelegate;
	}
		
	private SensorPeer peer;
	/**	Returns the Peer this Sensor actually uses.
		*/
	protected SensorPeer getPeer() {
		return peer;
	}
	
	private SensorPort port;
	public Port getPort() {
		return port;
	}
	
	private boolean peerDeliversFloatingPoint;
	/** Tells whether this Sensor delivers discrete values or not. A result of false means that the result of a getValue() can be cast to int without a loss of precision. The result of this method is only valid as long as no one of the conversion delegates changes.
		@see #convertToIncoming(double)
		@see #setIncomingConversionDelegate(Conversion)
		@see #setOutgoingConversionDelegate(Conversion)
		*/
	public boolean getDeliversFloatingPoint() {
		boolean result;
		
		Conversion localIncomingConversion = incomingConversionDelegate;
		Conversion localOutgoingConversion = outgoingConversionDelegate;
		
		if(localIncomingConversion == null
				&& localOutgoingConversion == null) {
			result = peerDeliversFloatingPoint;
		} else {
			if(localOutgoingConversion != null) {
				result = localOutgoingConversion
								.getDeliversFloatingPoint();
			} else {
				result = localIncomingConversion
								.getDeliversFloatingPoint();
			}
		}
		return result;
	}
	
	// whether we should try to cache values from the peer to minimize traffic
	private boolean peerCacheEnabled = true;
	/**	Should this Sensor try to minimize traffic to and from the Peer?
		This is enabled by default. If it is enabled this Sensor caches the values it receives from the Peer. Especially when all change-events are enabled it is assumed that the value of the last received event is still correct and the Peer will not be asked for it again.
		@see #getPeerCacheEnabled()
		@see #setChangeEventsEnabled(boolean)
		*/
	public void setPeerCacheEnabled(boolean enabled) {
		this.peerCacheEnabled = enabled;
	}
	/**	Tells whether this Sensor tries to minimize traffic to and from the Peer.
		@see #setPeerCacheEnabled(boolean)
		*/
	public boolean getPeerCacheEnabled() {
		return peerCacheEnabled;
	}
	
	private double peerDoubleCache; // what the peer sent us last time
	// (validity depends on peerCacheValid and useability on peerCacheEnabled
	
	// what we converted out of the peers last value
	private double incomingDoubleCache;
	// valid if:
	// isIncomingConversionCacheable() && peerCacheValid && peerCacheEnabled

	// what we send out last time
	private double outgoingDoubleCache; // valid if isOutgoingConversionCacheable() && peerCacheValid && peerCacheEnabled
	
	// this must never be true when changeEventsEnabled is false
	private boolean peerDoubleCacheValid;
	// tells whether we already did a conversion (gets false if we get a new conv. set)
	private boolean incomingDoubleCacheValid;
	// tells whether we already did a conversion (gets false if we get a new conv. set)
	private boolean outgoingDoubleCacheValid;

	private Object cacheMonitor = new Object();
	
	// this must never be false and a cache considered to be valid
	private boolean changeEventsEnabled;
	/**	Tells whether this Sensor is configured to receive Events from its Peer whenever something changes.
		If this is true the caching of peer-side values may work.
		@see #setPeerCacheEnabled(boolean)
		@see #setChangeEventsEnabled(boolean)
		*/
	protected boolean getChangeEventsEnabled() {
		return changeEventsEnabled;
	}
	/**	Configures whether this Sensor wants to receive Events from its Peer whenever something changes.
		This must be set to true for the caching of peer-values to work. It is false by default.
		@see #setPeerCacheEnabled(boolean)
		@see #getChangeEventsEnabled()
		*/
	protected void setChangeEventsEnabled(boolean enabled) {
		synchronized(cacheMonitor) {
			changeEventsEnabled = enabled;
			SensorPeer localPeer = peer;
			if(!enabled) {
				peerDoubleCacheValid = false;
				if(localPeer != null) {
					localPeer.disableEvent(propertyChangeEventDescriptor);
				}
			} else {
				if(localPeer != null) {
			// the Peer enables the described event as specific as possible
			// but it might be a super- or even a subset
					localPeer.enableEvent(propertyChangeEventDescriptor);
				}
			}
		}
	}
	
/*****	The value-retrieving methods									*****/

	public int getIntValue() {
		return (int)getValue();
	}
	
	public double getValue() {		
		if(debug.debug) {
			debug.printInfo("been asked to return value");
		}
		synchronized(cacheMonitor) {
			double oldPeerDouble = peerDoubleCache;
			double oldIncomingDouble = incomingDoubleCache;
					
			if(!(peerCacheEnabled && peerDoubleCacheValid)) {
				// ask peer for value
				SensorPeer localPeer = peer;
				if(isConnected() && localPeer != null) {
					if(debug.debug) {
						debug.printInfo("calling peer " +peer
										+" (peerCacheEnabled=" +peerCacheEnabled
										+", peerDoubleCacheValid="
										+peerDoubleCacheValid +")");
					}
					peerDoubleCache = localPeer.getValue();
					if(changeEventsEnabled) {
						peerDoubleCacheValid = true;
					}
				} else {
					throw new NotConnectedException("Can't read value.");
				}
			}

			// convert peerValue if different from last or
			// conversion not cacheable
			if(oldPeerDouble != peerDoubleCache
					|| !isIncomingConversionCacheable()
					|| !incomingDoubleCacheValid) {
				incomingDoubleCache = convertToIncoming(peerDoubleCache);
				if(debug.debug) {
					debug.printInfo("got " +peerDoubleCache
									+" from peer, computed incomingCache="
									+incomingDoubleCache);
				}
			}
			incomingDoubleCacheValid = true;

			// convert incomingValue if different from last or
			// conversion not cacheable
			if(oldIncomingDouble != incomingDoubleCache
					|| !isOutgoingConversionCacheable()
					|| !outgoingDoubleCacheValid) {
				outgoingDoubleCache = convertToOutgoing(incomingDoubleCache);
			}
			outgoingDoubleCacheValid = true;

			if(debug.debug) {
				debug.printInfo("returning " +outgoingDoubleCache);
			}
			return outgoingDoubleCache;
		}
	}
	
	public synchronized void disconnect() {
		if(isConnected()) {
			SensorPeer localPeer = peer;
			SensorPort localPort = port;
			peer = null;
			port = null;
			
			setChangeEventsEnabled(false);
			localPeer.removePropertyChangeListener(this);
			if(localPort == null) {
				if(debug.debug) {
					debug.printError("Have no port! Why do I have a peer then? Peer is "
								+localPeer);
				}
			}
		}
	}
	
	public boolean isConnected() {
		SensorPeer localPeer = peer;
		if(localPeer != null) {
			return localPeer.isConnected();
		} else {
			return false;
		}
	}

	public synchronized void connectWith(Port port) {
		if(port instanceof SensorPort) {
			connectWith((SensorPort) port);
		} else {
			throw new IllegalArgumentException("The given port is no SensorPort, it is " +((port==null) ? port+"" : port.getClass()+""));
		}
	}
	
	/** Connects this Sensor with the given Port.
		The Sensor gets its peer from the Port so it is therefore required before the Sensor can be properly used.
		@see SensorPort
		@see #connectWith(Port)
		*/
	public synchronized void connectWith(SensorPort port) {
		if(isConnected()) {
			disconnect();
		}
		
		peer = port.getPeerFor(this);
		this.port = port;
		
		peerDeliversFloatingPoint = peer.getDeliversFloatingPoint();
		peer.addPropertyChangeListener(this);

		Set allDescriptors = descriptorToListenerSetMap.keySet();
		Iterator i = allDescriptors.iterator();
		while(i.hasNext()) {
			SensorEventDescriptor descriptor = 
					(SensorEventDescriptor) i.next();
			if(debug.debug) {
				debug.printInfo("connected with new peer " +peer
								+". Enabling " +descriptor +" for listenerSet "
								+descriptorToListenerSetMap.get(descriptor));
			}
			peer.enableEvent(descriptor);
		}
		if(changes.hasListeners("value")) {
			if(debug.debug) {
				debug.printInfo("connected with new peer " +peer
								+". Enabling changeEvents because listeners are already added.");
			}
			setChangeEventsEnabled(true);
		} else {
			if(debug.debug) {
				debug.printInfo("connected with new peer " +peer
								+". NOT enabling changeEvents because no listeners available by now.");
			}
		}
	}	

/*****	Now comes the stuff for the events and their listeners			*****/

	protected PropertyChangeSupport changes =
				new PropertyChangeSupport(this);
	public void addPropertyChangeListener(PropertyChangeListener l) {
		changes.addPropertyChangeListener(l);
		try {
			setChangeEventsEnabled(true);
		} catch(NullPointerException npe) {
			if(debug.debug) {
				debug.printError("Caught Exception, peer=" +peer +":");
				npe.printStackTrace();
			}
		}
	}
	public void removePropertyChangeListener(PropertyChangeListener l) {
		changes.removePropertyChangeListener(l);
		// disable Events if no Listeners remain ...
		synchronized(changes) {
			if(!changes.hasListeners("dummyProperty")) {
				try {
					setChangeEventsEnabled(false);
				} catch(NullPointerException npe) {
					if(debug.debug) {
						debug.printError("Caught Exception, peer=" +peer +":");
						npe.printStackTrace();
					}
				}
			}
		}
	}
	
	private Map	listenerToAdapterMapMap = new Hashtable();
	private Map	descriptorToListenerSetMap = new Hashtable();
		
/** Adds the specified SensorListener to this Sensor so that it receives the SensorEvents which are described by the specified SensorEventDescriptor. If listener is null this method performs no function, nor does it throw an exception. If the EventDescriptor is null the call is forwarded to the method <code>addSensorListener(SensorListener)</code>.
Note: By now using special SensorEventDescriptors only has the effect that the Events not described are discarded before delivering them to listener but they are still generated.
	@see #addSensorListener(SensorListener)
	@see #removeSensorListener(SensorListener,SensorEventDescriptor)
	@see SensorListener
	@see SensorEventDescriptor
*/
	public synchronized void
			addSensorListener(final SensorListener listener,
							final SensorEventDescriptor eventDescriptor)
	{
		if(eventDescriptor == null) {
			addSensorListener(listener);
			return;
		}
		if(listener == null) {
			return;
		}
		Map adapterMap = (Map)listenerToAdapterMapMap.get(listener);
		if(adapterMap != null) {
			if(adapterMap.containsKey(eventDescriptor)) {
				return;
			}
		} else {
			adapterMap = new Hashtable();
		}
		
		PropertyChangeListener adapter =
			new PropertyChangeListener()
			{				
			// this is an adapter-class between propertyChangeEvents
			// and sensor-Events. This class does for now the filtering
			// with the eventDescriptor
					public void propertyChange(PropertyChangeEvent event) {
						SensorEvent sEvent = (SensorEvent)event.getNewValue();
						if(eventDescriptor.describes(sEvent)) {
							listener.processEvent(sEvent);
						}
					}
			};
			
		adapterMap.put(eventDescriptor, adapter);
		listenerToAdapterMapMap.put(listener, adapterMap);
		
		// that also enables changeEvents by now
		addPropertyChangeListener(adapter);
	}
	
/** Removes the specified SensorListener so that it no longer receives from this Sensor the SensorEvents which are described by the specified SensorEventDescriptor. This method performs no function, nor does it throw an exception, if the listener specified by the argument was not previously added with the specified EventDescriptor to this Sensor. If listener or the EventDescriptor is null, no exception is thrown and no action is performed.
	Note that after a call to this method, it is still possible that the specified SensorListener receives a SensorEvent which is described by the specified SensorEventDescriptor. That is the case if the intersection of the events described by the given SensorEventDescriptor, with the events described by another SensorEventDescriptor, for which the specified SensorListener is still registered as a listener, is not empty.
	@see #addSensorListener(SensorListener,SensorEventDescriptor)
	@see SensorListener
	@see SensorEventDescriptor
*/
	public synchronized void removeSensorListener(
						SensorListener listener,
						SensorEventDescriptor eventDescriptor)
{
		if(listener == null || eventDescriptor == null) {
			if(debug.debug) {
				debug.printInfo("Request to remove "
							+listener
							+" for "
							+eventDescriptor
							+". Not removing.");
			}
			return;
		}
		
		Map adapterMap = (Map)listenerToAdapterMapMap.get(listener);
		if(adapterMap != null) {
			PropertyChangeListener adapter =
					(PropertyChangeListener) adapterMap.get(eventDescriptor);
			if(adapter != null) {
				// the adapter has to be removed first from the
				// PropertyChangeSupport
				// so that any PropertyChangeEvents aren't delivered anymore
				changes.removePropertyChangeListener(adapter);
				
				adapterMap.remove(eventDescriptor);
			}
			// remove listener from Map if its adapterMap is empty
			// because if there is no adapter, there will be no delivered Event
			// anymore (in this implementation!)
			if(adapterMap.isEmpty()) {
				listenerToAdapterMapMap.remove(listener);
			}
		}
		
		Set listenerSet = (Set)descriptorToListenerSetMap.get(eventDescriptor);
		if(listenerSet == null || !listenerSet.remove(listener)) {
			if(debug.debug) {
				debug.printInfo("Request to remove "
							+listener
							+" for "
							+eventDescriptor
							+". Not removing, "
							+(listenerSet==null?"descriptor":"event")
							+" is not registered.");
			}
			return;
		}
		// remove descriptor from Map if its listenerSet is empty
		if(listenerSet.isEmpty()) {
			descriptorToListenerSetMap.remove(eventDescriptor);
			// disable because we removed the last listener for this descriptor
			// no peer-side event-filtering by now
			/*try {
				peer.disableEvent(eventDescriptor);
			} catch(NullPointerException npe) {
				if(debug) {
					printError("Caught Exception, peer=" +peer +":");
					npe.printStackTrace();
				}
			}*/
		}
		
	}
	
	/**	An instance of <code>AllSensorEventDescriptor</code>. It is used as a descriptor for PropertyChangeEvents.
		@see SensorEventDescriptor
		@see AllSensorEventDescriptor
		*/
	protected SensorEventDescriptor propertyChangeEventDescriptor = new AllSensorEventDescriptor();
	public synchronized void addSensorListener(SensorListener listener) {
		// default: listener gets notified whenever a change occurs,  expensive!
		addSensorListener(listener, propertyChangeEventDescriptor);
	}
	
	public synchronized void removeSensorListener(SensorListener listener) {
		// Is it better to maintain another Map for listener->descriptors
		// because this is too slow?
		Set allDescriptors = descriptorToListenerSetMap.keySet();
		Iterator i = allDescriptors.iterator();
		while(i.hasNext()) {
			SensorEventDescriptor descriptor = 
					(SensorEventDescriptor) i.next();
			Set listenerSet = (Set) descriptorToListenerSetMap.get(descriptor);
			if(listenerSet.contains(listener)) {
				removeSensorListener(listener, descriptor);
			}
		}
	}
	
	public void propertyChange(PropertyChangeEvent event) {
		String propName = event.getPropertyName();
		if(propName.equals("value") && event.getSource() == peer) {
			SensorEvent newEvent = (SensorEvent)event.getNewValue();
			SensorEvent myNewEvent;
			// change the source of the Event to this Sensor
			// and convert Data as needed
			synchronized(cacheMonitor) {
				double oldPeerDouble = peerDoubleCache;
				double oldIncomingDouble = incomingDoubleCache;
				double oldOutgoingDouble = outgoingDoubleCache;

				if(!(peerCacheEnabled && peerDoubleCacheValid)) {
					// either we don't have the old value
					// or we shouldn't use it
					oldPeerDouble = newEvent.getOldValue();

					if(debug.debug) {
						if(peerDoubleCacheValid
								&& peerDoubleCache != oldPeerDouble) {
							debug.printError("peerDoubleCache="
											+peerDoubleCache
											+"!="
											+oldPeerDouble +"=eventOldDouble");
						}
					}

					if(changeEventsEnabled) {
						peerDoubleCacheValid = true;
					}
				}

				if(debug.debug) {
					if(newEvent.isFloatingPoint()
							!= peerDeliversFloatingPoint) {
						debug.printError("eventFP="
										+newEvent.isFloatingPoint()
										+"!="
										+peerDeliversFloatingPoint
										+"=peerDeliversFloatingPoint"
										+", peer=" +peer
										+", SensorEventSource="
										+newEvent.getSource());
					}
				}

				// if the old Value has "changed", we also need a new
				// conversion
				// we don't convert if the values are the same
				// even if !isIncomingConversionCacheable() because
				// old Value should normally be what was the value
				// _at that time_
				if(oldPeerDouble != peerDoubleCache
						|| !incomingDoubleCacheValid) {
					oldIncomingDouble = convertToIncoming(oldPeerDouble);

					if(oldIncomingDouble != incomingDoubleCache
							|| !outgoingDoubleCacheValid) {
						oldOutgoingDouble =
								convertToOutgoing(oldIncomingDouble);
					}
				}
				
				peerDoubleCache = newEvent.getNewValue();
				// the new value is assumed to be different from the last one
				// because otherwise we (usually) wouldn't have an event
				incomingDoubleCache = convertToIncoming(peerDoubleCache);
				incomingDoubleCacheValid = true;
				// but the outgoing conversion might not be
				// necessary
				if(oldIncomingDouble != incomingDoubleCache
						|| !isOutgoingConversionCacheable()
						|| !outgoingDoubleCacheValid) {
					outgoingDoubleCache =
							convertToOutgoing(incomingDoubleCache);
				}
				outgoingDoubleCacheValid = true;
				
				
				if(debug.debug) {
					debug.printInfo("computed value="
								+outgoingDoubleCache
								+"(old=" +oldOutgoingDouble +")"
								+" for peer=" +peerDoubleCache
								+"(old=" +oldPeerDouble
								+")");
				}
				
				myNewEvent =
						new SensorEvent(this,
										newEvent.getEventDescriptor(),
										newEvent.getTimestamp(),
										oldOutgoingDouble,
										outgoingDoubleCache,
										getDeliversFloatingPoint(),
				// processEvent() may fire a propertyChangeEvent of its own.
				// according to the documentation of
				// PropertyChangeEvent.getPropagationID()
				// it is then required that this new event has the
				// same propagationId as the one catched here
				// because of a possible future use after beans 1.0
										event.getPropagationId());
			}
		
			processEvent(myNewEvent);
		}
	}
	
	private SensorEvent lastEvent;
	private Object lastEventMonitor = new Object();
	public void processEvent(SensorEvent event) {
		SensorEvent last = null;
		// make a copy to be thread-safe without too much blocking
		// but as long as firePropertyChange isn't synchronized
		// it is not guaranteed that the Events come in the right order
		synchronized(lastEventMonitor) {
			last = lastEvent;
			lastEvent = event;
		}
		
		if(changes.hasListeners("value")) {
			PropertyChangeEvent pce =
					new PropertyChangeEvent(this,
											"value",
											last,
											event);
			pce.setPropagationId(event.getPropagationId());
			if(debug.debug) {
				debug.printInfo("firing pce with old("
								+last
								+"), new("
								+event +")"
								+" from " +event.getSource());
			}
			changes.firePropertyChange(pce);
		}
	}

	private SensorListener dummyListener = new SensorListener() {
					public void processEvent(SensorEvent se) {
					}
				};
	/**	Enables the type of Event described by the given SensorEventDescriptor.
		This feature is not fully implemented in the sense that it can't be used to specify which events should be generated and which should not. Events can only be enabled once and wiil for now usually never be disabled. Also if events are enabled for any SensorEventDescriptor, they get enabled as if an instance of <code>AllSensorEventDescriptor</code> has been used. It can however be used to enable the caching of the peer-side values without registering a listener for this Sensor.
		@see #setPeerCacheEnabled(boolean)
		@see #setChangeEventsEnabled(boolean)
		@see AllSensorEventDescriptor
		*/
	public void enableEvent(SensorEventDescriptor eventDescriptor) {
		addSensorListener(dummyListener, eventDescriptor);
	}
	
	/**	Disables the type of Event described by the given SensorEventDescriptor.
		@see #enableEvent(SensorEventDescriptor)
		*/
	public void disableEvent(SensorEventDescriptor eventDescriptor) {
		removeSensorListener(dummyListener, eventDescriptor);
	}

	private SensorEventDescriptor updateSensorEventDescriptor =
											new AllSensorEventDescriptor();
	// update is forwarded to the peer
	// because it should better be delivered by the
	// real event-delivering thread
	public synchronized void update() {
		if(peer != null) {
			peer.update();
		}
	}
	
	/** A SensorPort for the value of this Sensor.
		An instance of this class can be retrieved by calling <code>getSensorPort()</code> but apart from using something not every Sensor has there should be no problem by using this class directly.
		@see #getSensorPort()
		*/
	public class ValueSensorPort extends BaseSensorPort {
		// maybe better throw notConnectedException when enclosing Sensor
		// has no port, so that we can use some reasonable values to
		// initialize the RawSensorPort?
		public ValueSensorPort() {
		}
		public SensorPeer getPeerFor(Sensor sensor) {
			return RawSensor.this;
		}
	}
	
	public SensorPort getSensorPort() {
		return new ValueSensorPort();
	}
}
