package de.jaetzold.art;

/** This class is for interfaces which are defined just by a single String.
	This is for now sufficient for all RobotInterface-Implementations in the first version of ART. A RobotInterface definition e.g. for LEGO MINDSTORMS on all known ports is created by a <code>new RobotInterfaceStringDefinition("MS")</code><br>
	Known String-ID's include
	<ul>
	<li> <code>MS </code> - Mindstorms Brick </li>
	<li> <code>CM </code> - Cybermaster </li>
	<li> <code>RCX</code> - like MS+CM </li>
	<li> <code>FT </code> - Fischertechnik </li>
	<li> <code>AWT</code> - special AWT-Interface (from the examples)</li>
	</ul>
	For the known implementations it is possible to append a desired serial port name directly, e.g. <code>MS/dev/ttyS0</code> or <code>RCXCOM2</code> or to append just numbers, like <code>FT3</code> to use all known names for the serial port which is <code>COM3</code> in Windows, plus the 3rd one in the list from JavaComm.
	*/
public class RobotInterfaceStringDefinition
		implements RobotInterfaceDefinition
{
	private String interfaceId;
	/** Returns the String used to construct this RobotInterfaceStringDefinition.
		*/
	public String getInterfaceString() {
		return interfaceId;
	}
	/**	Construct a new RobotInterfaceDefinition from the String given by interfaceId.
		*/
	public RobotInterfaceStringDefinition(String interfaceId) {
		if(interfaceId != null) {
			this.interfaceId = interfaceId;
		} else {
			this.interfaceId = "ALL";
		}
	}
	
	public boolean equals(Object o) {
		return	o.getClass() == this.getClass()
				&& ((RobotInterfaceStringDefinition)o)
						.interfaceId.equals(interfaceId);
	}
	
	/**	Returns the String used to construct this Object preceded by "Definition for interface ".
		*/
	public String toString() {
		return "Definition for interface " +interfaceId;
	}
}
