package de.jaetzold.art;

import de.jaetzold.util.EventBase;

public class SensorEvent extends EventBase
{
	public boolean equals(Object o) {
		boolean result = false;
		if(o!=null && this.getClass() == o.getClass()) {
			SensorEvent se = (SensorEvent) o;
			if(timestamp == se.timestamp
					&& source.equals(se.source)
					&& isFloatingPoint == se.isFloatingPoint
					//&& isBoolean == se.isBoolean
					&& doubleCache == se.doubleCache
					&& oldDouble == se.oldDouble
					//&& (isBoolean ? booleanCache == se.booleanCache : true)
					//&& (isBoolean ? oldBoolean == se.oldBoolean : true)
					&& eventDescriptor.equals(se.eventDescriptor)) {
				result = true;
			}
		}

		if(debug) {
			printInfo("tested for equality:"
						+this
						+(result?"":"!")
						+"=" +o);
		}
		return result;
	}
	
	private double doubleCache;
	private double oldDouble;
	private boolean isFloatingPoint;
	
	public int getIntValue() {
		return (int)doubleCache;
	}
	public double getValue() {
		return doubleCache;
	}

	public long getNewIntValue() {
		return getIntValue();
	}
	public double getNewValue() {
		return getValue();
	}

	public int getOldIntValue() {
		return (int)oldDouble;
	}
	public double getOldValue() {
		return oldDouble;
	}
	
	public boolean isFloatingPoint() {
		return isFloatingPoint;
	}
	
	protected long timestamp;
	public long getTimestamp() {
		return timestamp;
	}
	
	protected SensorEventDescriptor eventDescriptor;
	public SensorEventDescriptor getEventDescriptor() {
		return eventDescriptor;
	}
	private void setEventDescriptor(SensorEventDescriptor eventDescriptor) {
		if(eventDescriptor == null) {
			eventDescriptor = new AllSensorEventDescriptor();
		}
		this.eventDescriptor = eventDescriptor;
	}
	
	protected Object propagationId;
	public Object getPropagationId() {
		return propagationId;
	}
	
	public SensorEvent(Object source,
				SensorEventDescriptor descriptor,
				long timestamp,
				double oldDoubleValue,
				double newDoubleValue,
				boolean isFloatingPoint,
				Object propagationId)
	{
		super(source);
		this.timestamp = timestamp;
		this.oldDouble = oldDoubleValue;
		this.doubleCache = newDoubleValue;
		this.isFloatingPoint = isFloatingPoint;
		if(propagationId == null) {
			this.propagationId = this;
		} else {
			this.propagationId = propagationId;
		}
		setEventDescriptor(descriptor);
	}
		
	private boolean bounce = false;
	public String toString() {
		if(debug && !bounce) {
			bounce=true;
			
			String tmp = super.toString();
			String header = "";
			String trailer = "";

			if(tmp.charAt(tmp.length()-1) == ']') {
				tmp = tmp.substring(0, tmp.length()-1);
				header = "[";
				trailer = "]";
			}

			printInfo("toString() of: "
					+header
					+tmp
					+", timestamp="
					+timestamp
					+", value="
					+doubleCache
					+"(old="
					+oldDouble
					+"), descriptor="
					+eventDescriptor
					+trailer);
		}
		bounce=false;
		
		String result = "";
		return (isFloatingPoint
					? doubleCache+""
					: (long)doubleCache+"");
	}
		
	// The rest of the code are just some standard-methods i commonly use for debugging-purposes:
	/** Name of the class, including package-name. Useful for knowing the classname even in static-methods. Used as the header for any properties.
		@see #debug
	*/
	private static String className = "de.jaetzold.art.SensorEvent";
	
	/** Print className, this.toString() and message on stderr
		@see #className
		@see #printErrorStatic(String)
		@see #printInfo(String)
		@see #printInfoStatic(String)
	*/
	private void printError(String message) {
		System.err.println(className +"(" +this +")" +": " +message);
	}
	/** Print className, this.toString() and message on stdout
		@see #className
		@see #printError(String)
		@see #printErrorStatic(String)
		@see #printInfoStatic(String)
	*/
	private void printInfo(String message) {
		System.out.println(className +"(" +this +")" +": " +message);
	}
	/** Print className and message on stderr
		@see #className
		@see #printError(String)
		@see #printInfo(String)
		@see #printInfoStatic(String)
	*/
	private static void printErrorStatic(String message) {
		System.err.println(className +": " +message);
	}
	/** Print className and message on stdout
		@see #className
		@see #printError(String)
		@see #printErrorStatic(String)
		@see #printInfo(String)
	*/
	private static void printInfoStatic(String message) {
		System.out.println(className +": " +message);
	}
	
	/** Reflects the value of the property className.debug at the time this class had been initialized. If such a property does not exist enclosing packages are tried for this property.
		<p>Example:
		If the package name is a.b and the class is called C, debug will be true if the property
		<ul>
		<li>a.b.C.debug exists and is true</li>
		<li>a.b.debug exists and is true and a.b.C.debug doesn't exist</li>
		<li>a.debug exists and is true and a.b.debug as well as a.b.C.debug don't exist</li>
		<li>debug exists and is true and a.debug as well as a.b.debug and a.b.C.debug don't exist</li>
		</ul>
		In any other case debug will be set to false.</p>
		@see #className
		@see #printError(String)
		@see #printErrorStatic(String)
		@see #printInfo(String)
		@see #printInfoStatic(String)
	*/
	private static boolean debug = false;
	static {
		String v = null;
		v = System.getProperty(className+".debug");
		int index = className.length();
		while(v==null && (index=className.lastIndexOf('.', index-1)) != -1) {
			v = System.getProperty(className.substring(0,index)+".debug");
		}
		if(v==null) {
			v = System.getProperty("debug");
		}
		
		// results to false if v!="true" (ignoring case)
		debug = Boolean.valueOf(v).booleanValue();
	}
}
