package de.jaetzold.art;

import de.jaetzold.art.platform.SensorPeer;

/**	A SensorPort is a Port which delivers SensorPeers for Sensors.
	*/
public interface SensorPort extends Port {
	// a Peer is considered "Free" if it has no PropertyChangeListeners
	/**	Returns a SensorPeer which can be used by the given Sensor.
		There should never be the need in calling this method from somewhere outside of a Sensor-implementation.
		@see #connectWith(Sensor)
		@see RawSensor#connectWith(SensorPort)
		*/
	public SensorPeer getPeerFor(Sensor sensor);
	// evtl. entfernen, denn was ist wenn ein Sensor mit einem Sensor
	// verbunden wird, wenn z.B. beide auch Ports sind?
	/**	Connects the given Sensor to this port.
		This gives the Sensor its Peer.
		If the Sensor does not allow this connection an Exception is thrown.
		This could be the case if the Sensor is already connected to a different Port, does'nt allow multiple connections and doesn't allow automatic disconnection.
		@see Sensor#connectWith(Port)
		*/
	public void connectWith(Sensor sensor);
	/**	Disconnects the given Sensor from this port.
		This removes the peer associated with this Port for the Sensor.
		If the Sensor is not connected or the Sensor or the Port doesn't allow disconnection an Exception is thrown.
		@see Sensor#disconnect()
		*/
	public void disconnect(Sensor sensor);
}
