package de.jaetzold.art;

import de.jaetzold.util.StateDecider;
import de.jaetzold.util.Debug;

/**	A StateDecider which delivers the value of the target-Sensor as the result to <code>nextStateValue(double,double)</code>.
	@see Sensor
	*/
public class SensorStateDecider implements StateDecider {
	private static final Debug debug =
								new Debug("de.jaetzold.art.SensorStateDecider");

	public boolean isCacheable() {
		return false;
	}
	
	private boolean events;
	private double eventValue;
	/**	The value returned is the one the target sensor currently has.
		*/
	public double nextStateValue(double state, double value) {
		double result;
		if(events) {
			result = eventValue;
		} else {
			result = targetSensor.getValue();
		}
		if(debug.debug) {
			debug.printInfo(" been asked nextStateValue("
							+state +", " +value +"), returning "
							+result +" (events=" +events +")"
							+", targetSensor.getIntValue()="
							+targetSensor.getIntValue());
		}
		
		return result;
	}
	
	private Sensor targetSensor;
	/**	Constructs a new StateDecider which always uses the value of the given Sensor as the next state.
		@see de.jaetzold.util.StateDecider#nextStateValue(double,double)
		*/
	public SensorStateDecider(Sensor targetSensor) {
		this(targetSensor, false);
	}
	/**	Constructs a new StateDecider which always uses the value of the given Sensor as the next state.
		If events is true a SensorListener is registered by the Sensor and the value of the last received SensorEvent is cached to be the next state-value.
		@see de.jaetzold.util.StateDecider#nextStateValue(double,double)
		*/
	public SensorStateDecider(Sensor targetSensor, boolean events) {
		this.targetSensor = targetSensor;
		this.events = events;
		if(events) {
			targetSensor.addSensorListener(
				new SensorListener() {
					{
						if(debug.debug) {
							debug.printInfo(" adding " +this
											+" as SensorListener for "
											+SensorStateDecider.this
												.targetSensor);
						}
					}
					public void processEvent(SensorEvent se) {
						if(debug.debug) {
							debug.printInfo(" got event " +se
											+" from " +se.getSource());
						}
						eventValue = se.getValue();
					}
				}
			);
		}
	}
}
