// BEGIN AWTRobotInterface:main
package de.jaetzold.art.examples;

// lots of imports
// ...
import java.awt.Frame;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.Label;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Map;
import java.util.Hashtable;
import java.util.Vector;

import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.platform.RobotInterfaceAdapter;
import de.jaetzold.art.RobotInterfaceDefinition;
import de.jaetzold.art.RobotInterfaceStringDefinition;
import de.jaetzold.art.Port;
import de.jaetzold.art.Sensor;
import de.jaetzold.art.platform.SensorPeer;
import de.jaetzold.art.SensorPort;
import de.jaetzold.art.SensorEvent;
import de.jaetzold.art.platform.BaseSensorPeer;
import de.jaetzold.art.platform.BaseSensorPort;
import de.jaetzold.art.Actuator;
import de.jaetzold.art.platform.ActuatorPeer;
import de.jaetzold.art.ActuatorPort;
import de.jaetzold.art.platform.BaseActuatorPort;
import de.jaetzold.util.Debug;
// ...

public class AWTRobotInterface extends RobotInterfaceAdapter {
	private static Debug debug =
		new Debug("de.jaetzold.art.platform.awt.AWTRobotInterface");

	protected TextField in0 = new TextField("0.0");
	protected TextField in1 = new TextField("0.0");
	protected TextField out0 = new TextField("0.0");
	protected TextField out1 = new TextField("0.0");

// RobotInterfaceDefinition:
// BEGIN definition	
	private static String definitionPrefix = "AWT";
	private RobotInterfaceDefinition definition;
	
	public RobotInterfaceDefinition getInterfaceDefinition() {
		return definition;
	}
	
	public boolean conformsTo(RobotInterfaceDefinition definition) {
		if(this.definition.equals(definition)
				|| new RobotInterfaceStringDefinition(definitionPrefix)
						.equals(definition)
				|| new RobotInterfaceStringDefinition("ALL")
						.equals(definition)) {
			return true;
		} else {
			return false;
		}
	}

	private AWTRobotInterface(int interfaceNumber) {
		String definitionString = definitionPrefix +interfaceNumber;
		definition =
			new RobotInterfaceStringDefinition(definitionString);
		// ...
		final Frame f = new Frame(getClass().getName()
									+" (" +definitionString +")");
		f.setLayout(new GridLayout(1,2));

		// one Panel for inputs, one for outputs
		Panel inputs = new Panel();
		Panel outputs = new Panel();
		inputs.setLayout(new BorderLayout());
		outputs.setLayout(new BorderLayout());
		inputs.add("North", new Label("Inputs:"));
		outputs.add("North", new Label("Outputs:"));
		f.add(inputs);
		f.add(outputs);
		
		// one Panel for the actual input-fields
		// and one for the output-fields
		Panel inputs2 = new Panel();
		Panel outputs2 = new Panel();
		inputs2.setLayout(new GridLayout(2,1));
		outputs2.setLayout(new GridLayout(2,1));
		inputs.add("Center", inputs2);
		outputs.add("Center", outputs2);
		
		// the input-fields
		Panel field = new Panel();
		inputs2.add(field);
		field.add(new Label("1:"));
		field.add(in0);
		
		field = new Panel();
		inputs2.add(field);
		field.add(new Label("2:"));
		field.add(in1);
		
		// the output-fields
		out0.setEditable(false);
		field = new Panel();
		outputs2.add(field);
		field.add(new Label("1:"));
		field.add(out0);
		
		out1.setEditable(false);
		field = new Panel();
		outputs2.add(field);
		field.add(new Label("2:"));
		field.add(out1);
		
		// unfortunately this creates a non-daemon Thread ...
		f.pack();
		f.show();
		// ...
	}
// END definition

// Factory-Method for creating an AWTRobotInterface:
// BEGIN getinterfaces
// ...
	private static Map portNamesToInterfaces = new Hashtable();
	/** Creates or just returns already existing instances of AWTRobotInterface for any matching Ports.
		*/
// ...
	public static synchronized RobotInterface[]
				getInterfaces(RobotInterfaceDefinition[] ports)
	{
		// 'search' port-array for the AWT-ones
		Vector interfaces = new Vector();
		for(int i=0; i<ports.length; i++) {
		// ...
			String interfaceString = null;
			if(ports[i] instanceof RobotInterfaceStringDefinition) {
				interfaceString =
					((RobotInterfaceStringDefinition)ports[i])
											.getInterfaceString();
		// ...
				// also accept String 'ALL', and therefore not only return an
				// AWTInterface if explicitly asked for
		// ...
				if(interfaceString.equals("ALL")) {
					interfaceString = "AWT";
				}
				if(interfaceString.startsWith(definitionPrefix)) {
					int interfaceNumber;
					if(interfaceString.equals(definitionPrefix)) {
						interfaceNumber = 0;
					} else {
						String numberString =
							interfaceString.substring(
								definitionPrefix.length());
						try {
							interfaceNumber =
								Integer.parseInt(numberString);
						} catch(NumberFormatException nfe) {
							continue;
						}
					}

					RobotInterface iface;
					iface = (RobotInterface)portNamesToInterfaces
									.get(new Integer(interfaceNumber));
					if(iface == null) {
						try {
		// ...
							iface =
								new AWTRobotInterface(interfaceNumber);
							// was initialization successful ?
							if(iface.isConnected()) {
								interfaces.add(iface);
								portNamesToInterfaces
									.put(new Integer(interfaceNumber),
											iface);
							} else {
								if(debug.debug) {
									debug.printInfo(iface
												+" is not connected,"
												+" not adding.");
								}
							}
		// ...
						} catch(Exception e) {
							if(debug.debug) {
								debug.printError("Got Exception " +e
												+" while trying to"
												+" open connection"
												+" to port "
												+interfaceNumber);
								debug.printError(
										debug.stackTraceToString(e));
							}
						}
					} else {
						interfaces.add(iface);
					}		
				}
			}
		// ...
		}
		
		RobotInterface[] result =
				new RobotInterface[interfaces.size()];
		interfaces.copyInto(result);
		return result;
	}
// END getinterfaces

// Factory-Methods for creating the Peers:
// BEGIN peer-creation
	private TextFieldSensorPeer inPeer0;
	private TextFieldSensorPeer inPeer1;
	SensorPeer getSensorPeer(SensorPort port, Sensor sensor) {
		switch(port.getPortNumber()) {
			case 0:
				if(inPeer0 == null) {
					inPeer0 = new TextFieldSensorPeer(in0);
				}
				return inPeer0;
			case 1:
				if(inPeer1 == null) {
					inPeer1 = new TextFieldSensorPeer(in1);
				}
				return inPeer1;
			default:
				return null;
		}
	}

	private TextFieldActuatorPeer outPeer0;
	private TextFieldActuatorPeer outPeer1;
	ActuatorPeer getActuatorPeer(ActuatorPort port, Sensor sensor) {
		switch(port.getPortNumber()) {
			case 0:
				if(outPeer0 == null) {
					outPeer0 = new TextFieldActuatorPeer(out0);
				}
				return outPeer0;
			case 1:
				if(outPeer1 == null) {
					outPeer1 = new TextFieldActuatorPeer(out1);
				}
				return outPeer1;
			default:
				return null;
		}
	}
// END peer-creation

// isConnected:
// BEGIN isConnected
	public boolean isConnected() {
		return true; // maybe use Frame.isDisplayable() ?
	}
// END isConnected

// isAlive:
// BEGIN isAlive
	public boolean isAlive() {
		return true;
	}
// END isAlive

// Factory-Methods for the Ports:
// BEGIN port-creation	
	public SensorPort[] getSensorPorts() {
		return new SensorPort[]{
						new AWTSensorPort(	"TextFieldInput0",
											0,
											Double.NEGATIVE_INFINITY,
											Double.POSITIVE_INFINITY,
											-1,
											null),
						new AWTSensorPort(	"TextFieldInput1",
											1,
											Double.NEGATIVE_INFINITY,
											Double.POSITIVE_INFINITY,
											-1,
											null)};
	}

	public ActuatorPort[] getActuatorPorts() {
		return new ActuatorPort[]{
						new AWTActuatorPort("TextFieldOutput0",
											0,
											"",
											Double.NEGATIVE_INFINITY,
											Double.POSITIVE_INFINITY,
											-1,
											null),
						new AWTActuatorPort("TextFieldOutput1",
											1,
											"",
											Double.NEGATIVE_INFINITY,
											Double.POSITIVE_INFINITY,
											-1,
											null)};
	}
// END port-creation

// Inner classes for the Ports:
// BEGIN AWTSensorPort	
	protected class AWTSensorPort extends BaseSensorPort {
		AWTSensorPort(String portName,
						int portNumber,
						double minimumValue,
						double maximumValue,
						long granularity,
						Port[] excludedPorts) {
			super(AWTRobotInterface.this,
					portName,
					portNumber,
					minimumValue,
					maximumValue,
					granularity,
					excludedPorts);
		}
		
		public SensorPeer getPeerFor(Sensor sensor) {
			SensorPeer peer = getSensorPeer(this, sensor);
			return peer;
		}
	}

	protected class AWTActuatorPort extends BaseActuatorPort {
		AWTActuatorPort(String portName,
						int portNumber,
						String type,
						double minimumValue,
						double maximumValue,
						long granularity,
						Port[] excludedPorts) {
			super(AWTRobotInterface.this,
					portName,
					portNumber,
					type,
					minimumValue,
					maximumValue,
					granularity,
					excludedPorts);
		}
		
		public SensorPeer getPeerFor(Sensor sensor) {
			ActuatorPeer peer = getActuatorPeer(this, sensor);
			return peer;
		}
		public ActuatorPeer getPeerFor(Actuator actuator) {
			ActuatorPeer peer = getActuatorPeer(this, actuator);
			return peer;
		}
	}
// END AWTSensorPort

// Inner class for the SensorPeer:
// maybe put outside of class because it is of more general use?
// BEGIN TextFieldSensorPeer
	protected class TextFieldSensorPeer extends BaseSensorPeer {
		protected TextField input;
		public TextFieldSensorPeer(TextField input) {
			this.input = input;
			
			input.addActionListener(
				new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						try {
							double oldValue = currentValue;
							currentValue = Double.parseDouble(
												ae.getActionCommand());
							
							processEvent(
								new SensorEvent(this,// source
											null,    // descriptor
											System.currentTimeMillis(),
											oldValue,
											currentValue,
											true,    // isFloatingPoint
											null)    // propagationId
							);
						} catch(NumberFormatException nfe) {
							TextFieldSensorPeer.this.input
								.setText("" +currentValue);
						}
					}
				}
			);
		}
		
		protected double currentValue;
		public double getValue() {
			return currentValue;
		}
		
		public boolean isConnected() {
			return AWTRobotInterface.this.isConnected();
		}
	}
// END TextFieldSensorPeer

// Inner class for the ActuatorPeer:
// BEGIN TextFieldActuatorPeer
	protected class TextFieldActuatorPeer   extends TextFieldSensorPeer
											implements ActuatorPeer
	{
		protected TextField output;
		public TextFieldActuatorPeer(TextField output) {
			super(output);
			this.output = output;
		}
		
		public void setValue(double value) {
			setValue(value, true);
		}
		public synchronized void setValue(	double value,
											boolean waitMode)
		{
			String tmp = "" +value;

			double oldValue = getValue();
			output.setText(tmp);
			// beware if the TextField is changed by someone else
			// in this moment
			if(oldValue != value) {
				ActionEvent ae =
					new ActionEvent(output,
									ActionEvent.ACTION_PERFORMED,
									tmp);
				// output.dispatchEvent(ae); // Don't let this Thread
											 // process the Event!

				// This way the Thread processing the Event is the
				// real AWT-Event-Thread:
				output.getToolkit().getSystemEventQueue()
												.postEvent(ae);
			}
		}
	}
// END TextFieldActuatorPeer
}
// END AWTRobotInterface:main
