// BEGIN DriveTrain:construction
package de.jaetzold.art.examples;

import de.jaetzold.art.Motor;
// ...
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
// ...
public class DriveTrainSimpleAlgorithm implements DriveTrain {
	public DriveTrainSimpleAlgorithm(Robot robot) {
		this.left = robot.getLeftMotor();
		this.right = robot.getRightMotor();
		setState(STOP);
		stop();
	}
// BEGIN DriveTrain:robot
	public static interface Robot {
		public Motor getLeftMotor();
		public Motor getRightMotor();
	}
// END DriveTrain:robot
// BEGIN DriveTrain:implementation
	protected Motor left;
	protected Motor right;
	
	protected int state;

	public void stop() {
		right.off();
		left.off();
		
		setState(state | STOP);
	}
	
	public void resume() {
		if(state != STOP) {
			right.on();
			left.on();

			setState(state & ~ STOP);
		}
	}

	public void forward() {
		right.forward();
		left.forward();
		
		setState(FORWARD);
		resume();
	}
// ...
	public void backward() {
		right.backward();
		left.backward();

		setState(BACKWARD);
		resume();
	}
// ...
	public void leftSpin() {
		right.forward();
		left.backward();
		
		setState(LEFT_SPIN);
		resume();
	}
// ...
	public void rightSpin() {
		right.backward();
		left.forward();
		
		setState(RIGHT_SPIN);
		resume();
	}
// ...
// END DriveTrain:implementation
// BEGIN DriveTrain:state
	protected PropertyChangeSupport changes =
				new PropertyChangeSupport(this);
	public void addPropertyChangeListener(PropertyChangeListener l) {
		changes.addPropertyChangeListener(l);
	}
	public void removePropertyChangeListener(PropertyChangeListener l) {
		changes.removePropertyChangeListener(l);
	}
	
	protected void setState(int state) {
		int previousState = this.state;
		this.state = state;
		changes.firePropertyChange("state", previousState, state);
	}
	public int getState() {
		return state;
	}
// END DriveTrain:state
// ...
	public DriveTrainSimpleAlgorithm() throws Exception {
		this((Robot)Class.forName(
				System.getProperty(
					className +".robotClass",
					"de.jaetzold.art.examples.DriveTrainSimpleRobot"
				)
			).newInstance()
		);
	}
	private static String className =
			"de.jaetzold.art.examples.DriveTrainSimpleAlgorithm";
	public static void main(String[] argv) throws Exception {
		int index = 0;
		if(argv.length > index) {
			System.setProperty(className +".robotClass", argv[index++]);
		}
		if(argv.length > index) {
			System.setProperty(argv[index-1] +".portName", argv[index++]);
		}
		DriveTrainSimpleAlgorithm dt = new DriveTrainSimpleAlgorithm();
		
		DriveTrainView view = new DriveTrainView(dt);
		DriveTrainCommands commands = new DriveTrainCommands(dt);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		f.add("North", view);
		f.add("South", commands);
		
		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
// END DriveTrain:construction
