// BEGIN DriveTrain:robot
package de.jaetzold.art.examples;

import de.jaetzold.art.Motor;
import de.jaetzold.art.ActuatorPort;
import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.RobotInterfaceDefinition;
import de.jaetzold.art.RobotInterfaceStringDefinition;

public class DriveTrainSimpleRobot
						implements DriveTrainSimpleAlgorithm.Robot
{
	protected Motor leftMotor;
	protected Motor rightMotor;
		
	public DriveTrainSimpleRobot(RobotInterface iface) {
		ActuatorPort[] ports = iface.getActuatorPorts();
		if(ports.length < 2) {
			throw new IllegalArgumentException("RobotInterface "
											+iface
											+" provides only "
											+ports.length
											+" Ports, minimum is 2.");
		}

		leftMotor = new Motor();
		leftMotor.connectWith(ports[0]);
		
		rightMotor = new Motor();
		rightMotor.connectWith(ports[1]);
	}
	
	public Motor getLeftMotor() {
		return leftMotor;
	}
	public Motor getRightMotor() {
		return rightMotor;
	}
	public DriveTrainSimpleRobot(RobotInterfaceDefinition definition) {
		this(new de.jaetzold.art.RobotInterfaceFactory()
									.getInterface(definition));
	}
	public DriveTrainSimpleRobot(String interfacePortName) {
		this(new RobotInterfaceStringDefinition(interfacePortName));
	}
	public DriveTrainSimpleRobot() {
		this(System.getProperty(className +".portName", "ALL"));
	}
	private static String className =
			"de.jaetzold.art.examples.DriveTrainSimpleRobot";
// ...
	public static void main(String[] argv) {
		DriveTrainSimpleRobot robot;
		if(argv.length > 0) {
			robot = new DriveTrainSimpleRobot(argv[0]);
		} else {
			robot = new DriveTrainSimpleRobot((String)null);
		}

		DriveTrain driveTrain = new DriveTrainSimpleAlgorithm(robot);
		
		DriveTrainView view = new DriveTrainView(driveTrain);
		DriveTrainCommands commands =
							new DriveTrainCommands(driveTrain);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		f.add("North", view);
		f.add("South", commands);
		
		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
// END DriveTrain:robot
