package de.jaetzold.art.examples;

import de.jaetzold.art.Servo;
import de.jaetzold.art.BooleanSensor;
import de.jaetzold.art.AngleSensor;
import de.jaetzold.art.Port;
import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.RobotInterfaceFactory;
// BEGIN lise:ftrobot
public class LiSeSimpleFTDemoRobot extends LiSeSimpleBaseRobot {
	public LiSeSimpleFTDemoRobot(RobotInterface ft) {
		// fake parameters and set them afterwards
		super(null, 0, 0, 0, 0);
		
		Port endSwitchPort = ft.getPort("E7");
		Port countSensorPort = ft.getPort("E8");
		Port motorPort = ft.getPort("M4");
		
		// gear ratio Tower:Sensor = 1:6
		double countsPerFullTurn = 48;
		AngleSensor turnSensor = new AngleSensor(countsPerFullTurn);
		
		int endSwitchDirection = 1;
		BooleanSensor endSwitch = new BooleanSensor();
		
		// Angles are measured in full turns
		double positiveBorder = 21/48.0;
		double negativeBorder = -20/48.0;
		double endSwitchAngle = positiveBorder;
		
		maxRightAngle = 0.33;
		maxLeftAngle = -0.25;
		searchSpeed = 1;
		followSpeed = 0.33;
		
		turnServo = new Servo(turnSensor,
								endSwitch,
								endSwitchAngle,
								positiveBorder,
								negativeBorder,
								endSwitchDirection);

		turnSensor.setCountDecider(
						new FTRotationCountStateDecider(turnServo));

		endSwitch.connectWith(endSwitchPort);
		turnSensor.connectWith(countSensorPort);
		turnServo.connectWith(motorPort);
		
		// let the servo be a bit more sloppy in positioning
		// (that (mostly) prevents it from occasionally moving
		// back and forth around the desired position)
		turnServo.setPrecision(turnServo.getPrecision()*2);
	}
// ...
	public LiSeSimpleFTDemoRobot(String interfacePortName) {
		this(new RobotInterfaceFactory().getInterface(interfacePortName));
	}
	private static String className =
			"de.jaetzold.art.examples.LiSeSimpleFTDemoRobot";
	public LiSeSimpleFTDemoRobot() {
		this(System.getProperty(className +".portName", "FT"));
	}
	public static void main(String[] argv) {
		if(argv.length > 0) {
			System.setProperty(className +".portName", argv[0]);
		}
		
		new LiSeSimpleFTDemoRobot().turnServo.calibrate();
		
		try {
			Thread.sleep(5000);
		} catch(InterruptedException ie) {
		}
	}
// ...
}
// END lise:ftrobot
