package de.jaetzold.art.examples;

import de.jaetzold.art.Servo;
import de.jaetzold.art.BooleanSensor;
import de.jaetzold.art.RawSensor;
import de.jaetzold.art.AngleSensor;
import de.jaetzold.art.Port;
import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.RobotInterfaceFactory;
import de.jaetzold.util.SingleRange;

public class LiSeSimpleMSDemoRobot extends LiSeSimpleBaseRobot {
	public LiSeSimpleMSDemoRobot(RobotInterface iface) {	
		super(null, 0, 0, 0, 0);	// fake parameters and set them afterwards
		Port endSwitchPort = iface.getPort("3");	// Rotation and Switch
		Port countSensorPort = iface.getPort("3");	// coexist on the same Port
		Port motorPort = iface.getPort("C");
		
		double countsPerFullTurn = 48;	// gear ratio Tower:Sensor = 1:3
		AngleSensor turnSensor = new AngleSensor(countsPerFullTurn);
		turnSensor.setReversed(true);	// count in the other direction
		
		BooleanSensor endSwitch = new BooleanSensor();
		// since the RotationSensor and the Switch are connected to the same
		// Port the BooleanSensor has to be conigured to handle the Raw value
		RawSensor endSwitchRaw = new RawSensor();
		endSwitch.connectWith(endSwitchRaw.getSensorPort());
		// first Range is for 0, second for 1 and third is hysteresis
		// these are non-standard RCX-values because Rotation-Sensors
		// can deliver values as low as 350
		endSwitch.setStateDecider(
			new RCXRawSwitchStateDecider(
				new SingleRange(240),
				new SingleRange(0, 220),
				new SingleRange(200, 260)
			));
		
		// Angles are measured in full turns
		double positiveBorder = 15/48.0;
		double negativeBorder = -18/48.0;
		double endSwitchAngle = positiveBorder;
		
		maxRightAngle = -0.25;
		maxLeftAngle = 0.25;
		searchSpeed = 1;
		followSpeed = 0.1;
		
		int endSwitchDirection = 1;
		turnServo = new Servo(turnSensor,
								endSwitch,
								endSwitchAngle,
								positiveBorder,
								negativeBorder,
								endSwitchDirection);
		// A non-hardware Servo sends messages to the motor
		// in the Event-Thread of the turnSensor. Better not block
		// that event-Thread too long which would be the case for Mindstorms
		turnServo.setWaitForCompletion(false);
		
		endSwitchRaw.connectWith(endSwitchPort);
		// it is important to connect the AngleSensor _after_ the Switch
		// to have the sensor configured to Rotation
		turnSensor.connectWith(countSensorPort);
		turnServo.connectWith(motorPort);
	}
// ...
	public LiSeSimpleMSDemoRobot(String interfacePortName) {
		this(new RobotInterfaceFactory().getInterface(interfacePortName));
	}
	private static String className =
			"de.jaetzold.art.examples.LiSeSimpleMSDemoRobot";
	public LiSeSimpleMSDemoRobot() {
		this(System.getProperty(className +".portName", "MS"));
	}
	public static void main(String[] argv) {
		/*if(argv.length > 0) {
			System.setProperty(className +".portName", argv[0]);
		}
		
		LiSeSimpleMSDemoRobot robot = new LiSeSimpleMSDemoRobot();*/
		RobotInterface iface = new RobotInterfaceFactory().getInterface("MS");

		LiSe lise = new LiSeSimpleAlgorithm(
						new LiSeSimpleMSDemoRobot(iface));
		
		LiSeView view = new LiSeView(lise);
		LiSeCommands commands = new LiSeCommands(lise);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		f.add("North", view);
		f.add("South", commands);
		
		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
