package de.jaetzold.art.examples;

import java.beans.PropertyVetoException;

/** This implementation just waits for a specific time. Other implementations could be based on Servo or StepperMotor if the Robot can provide them.
	*/
public class NormalizedDriveTrainSimpleAlgorithm
							extends DriveTrainSimpleAlgorithm
							implements NormalizedDriveTrain
{
	protected int meterMilliseconds;
	protected int fullTurnMilliseconds;
	
	public NormalizedDriveTrainSimpleAlgorithm(Robot robot) {
		super(robot);
		meterMilliseconds = robot.getMeterMilliseconds();
		fullTurnMilliseconds = robot.getFullTurnMilliseconds();
	}
// BEGIN normalized:impl:robot		
	public interface Robot extends DriveTrainSimpleAlgorithm.Robot {
		public int getMeterMilliseconds();
		public int getFullTurnMilliseconds();
	}
// END normalized:impl:robot		
// BEGIN normalized:impl	
	public void forward(double meters) {
		if(meters<0) {
			backward(meters*-1);
		}
		forward();
		try {
			Thread.sleep((long)(meterMilliseconds*meters));
		} catch(InterruptedException ie) {
		}
		stop();
	}
// END normalized:impl		
	public void backward(double meters) {
		if(meters<0) {
			forward(meters*-1);
		}
		
		backward();
		try {
			Thread.sleep((long)(meterMilliseconds*meters));
		} catch(InterruptedException ie) {
		}
		stop();
	}
	
	public void leftSpin(double turns) {
		if(turns<0) {
			rightSpin(turns*-1);
			return;
		}
		
		leftSpin();
		try {
			Thread.sleep((int)(fullTurnMilliseconds*turns));
		} catch(InterruptedException ie) {
		}
		stop();
	}
	public void rightSpin(double turns) {
		if(turns<0) {
			leftSpin(turns*-1);
			return;
		}
		
		rightSpin();
		try {
			Thread.sleep((int)(fullTurnMilliseconds*turns));
		} catch(InterruptedException ie) {
		}
		stop();
	}
// ...
	public NormalizedDriveTrainSimpleAlgorithm() throws Exception {
		this((Robot)Class.forName(
				System.getProperty(
					className +".robotClass",
				"de.jaetzold.art.examples.NormalizedDriveTrainSimpleBaseRobot"
				)
			).newInstance()
		);
	}
	private static String className =
			"de.jaetzold.art.examples.NormalizedDriveTrainSimpleAlgorithm";
	public static void main(String[] argv) throws Exception {
		int index = 0;
		if(argv.length > index) {
			System.setProperty(className +".robotClass", argv[index++]);
		}
		if(argv.length > index) {
			System.setProperty(argv[index-1] +".portName", argv[index++]);
		}
		if(argv.length > index) {
			System.setProperty(argv[index-2] +".meterMilliseconds",
								argv[index++]);
		}
		if(argv.length > index) {
			System.setProperty(argv[index-3] +".fullTurnMilliseconds",
								argv[index++]);
		}
		NormalizedDriveTrainSimpleAlgorithm dt =
							new NormalizedDriveTrainSimpleAlgorithm();
		
		DriveTrainView view = new DriveTrainView(dt);
		
		NormalizedDriveTrainCommands commands =
							new NormalizedDriveTrainCommands(dt);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		f.add("North", view);
		f.add("South", commands);
		
		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
