package de.jaetzold.art.examples;

import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.RobotInterfaceStringDefinition;
import de.jaetzold.art.SensorPort;
import de.jaetzold.art.BooleanSensor;

public class SimpleTrustyRobot implements SimpleTrusty.Robot {
	protected DriveTrain driveTrain;
	protected BooleanSensor leftSensor;
	protected BooleanSensor rightSensor;
	protected int backupTime;
	protected int turnTime;
	
	public SimpleTrustyRobot(DriveTrain driveTrain,
							int backupTime,
							int turnTime,
							RobotInterface iface) {
		init(driveTrain, backupTime, turnTime, iface);
	}

	public SimpleTrustyRobot() {
		RobotInterface iface =
				new de.jaetzold.art.RobotInterfaceFactory()
					.getInterface(new RobotInterfaceStringDefinition("ALL"));
		
		DriveTrain driveTrain = new DriveTrainSimpleAlgorithm(
										new DriveTrainSimpleRobot(iface));
		
		init(driveTrain, 1500, 1000, iface);
	}
	
	private void init(	DriveTrain driveTrain,
						int backupTime,
						int turnTime,
						RobotInterface iface) {
		
		this.driveTrain = driveTrain;
		this.backupTime = backupTime;
		this.turnTime = turnTime;
		
		SensorPort[] ports = iface.getSensorPorts();
		if(ports.length < 2) {
			throw new IllegalArgumentException("RobotInterface " +iface
												+" provides only "
												+ports.length
												+" SensorPorts"
												+", minimum is 2.");
		}
		
		leftSensor = new BooleanSensor();
		leftSensor.connectWith(ports[0]);
		
		rightSensor = new BooleanSensor();
		rightSensor.connectWith(ports[1]);
	}

	public DriveTrain getDriveTrain() {
		return driveTrain;
	}
	public BooleanSensor getLeftBumper() {
		return leftSensor;
	}
	public BooleanSensor getRightBumper() {
		return rightSensor;
	}
	public int getBackupTime() {
		return backupTime;
	}
	public int getTurnTime() {
		return turnTime;
	}

	public static void main(String[] argv) {
		new SimpleTrusty(new SimpleTrustyRobot());
		
		try {
			Thread.sleep(20000);
		} catch(InterruptedException ie) {
		}
	}
}
