package de.jaetzold.art.examples;

import de.jaetzold.util.Task;
import de.jaetzold.util.Event;
import de.jaetzold.util.BooleanConversion;
import de.jaetzold.art.BooleanSensor;
import de.jaetzold.art.AngleSensor;
import de.jaetzold.art.subsumption.SchedulerTask;
// ...
import de.jaetzold.util.BeanSupport;	// does some reflect-stuff
import java.beans.PropertyChangeListener;
// ...
// BEGIN ariadne:sub
public class SubsumptionAriadne extends     SubsumptionTrusty
								implements  Ariadne,
											Trusty
{
	protected Ariadne ariadne;
	public SubsumptionAriadne(final Robot robot) {
		super(robot);
		this.ariadne = robot.getAriadne();
		final LiSe lise = robot.getLiSe();
		final AngleSensor liseAngle = lise.getAngleSensor();

		// construct orient-behavior
		// behavior and activator need to exchange information
		SchedulerTask.Entry entry = new SchedulerTask.Entry() {
// BEGIN ariadne:sub:behavior
			double angle;
			long lastOrientMillis;
			boolean oneShot;
			Object oneShotMonitor = new Object();
			Task behavior = new Task() {
				public Event perform() {
					double localAngle;
					synchronized(oneShotMonitor) {
						// be sure to only orient once by each angle
						if(!oneShot) {
							return null;
						}
						lastOrientMillis = System.currentTimeMillis();
						localAngle = angle;
						oneShot = false;
					}
					ariadne.orientByAngle(localAngle);
					// since it depends on some values
					// (lastOrientMillis, oneShot) it is better 
					// not to assume that there will be an event
					// from liseAngle soon enough
					activator.update();
					
					return null;
				}
			};
// END ariadne:sub:behavior
			BooleanSensor activator = new BooleanSensor();
			{
				activator.connectWith(liseAngle.getSensorPort());
				activator.setBooleanConversionDelegate(
					new BooleanConversion() {
						public boolean convertToBoolean(double value) {
							synchronized(oneShotMonitor) {
								// only if lise sees something at all:
								if(!lise.isSeeing()) {
									return false;
								}
						// Future Idea: don't orient if lise sees
						// light inthe direction we already are
						// or only orient if the seeing-angle
						// differs enough from the orient-to-angle (0)
								if((System.currentTimeMillis()
											- lastOrientMillis) >=
										robot.getMinOrientInterval())
								{
									angle = value;
									oneShot = true;
								}
								return oneShot;
							}
						}
						public boolean isCacheable() {
							return false;
						}
					}
				);
			}
			// ...
			public BooleanSensor getBooleanSensor() {
				return activator;
			}
			public Task getBehavior() {
				return behavior;
			}
			public Comparable getPriority() {
				return new Double(0.5);
			}
			// ...
		};
		// insert orient-behavior into scheduler of superclass:
		scheduler.add(entry);
	}
// BEGIN ariadne:sub:robot
	public interface Robot extends SubsumptionTrusty.Robot {
		public Ariadne getAriadne();
		public LiSe getLiSe();
		public long getMinOrientInterval();
	}
// END ariadne:sub:robot
// ...	
	public void orientByAngle(double angle) {
		stop();
		ariadne.orientByAngle(angle);
	}
// ...
}
// END ariadne:sub
