package de.jaetzold.art.examples;

import de.jaetzold.art.RobotInterfaceFactory;
import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.Port;
import de.jaetzold.util.NotConnectedException;

public class SubsumptionAriadneFTDemoRobot extends SubsumptionAriadneRobot
{
	// only needed for (future) subclasses and test-main
	protected NormalizedDriveTrain driveTrain;
	
	public SubsumptionAriadneFTDemoRobot() {
		super(null, null, 0, null);	// fake parameters, set them afterwards
		
		RobotInterface iface = new RobotInterfaceFactory().getInterface("FT");
		if(iface == null) {
			throw new NotConnectedException("No Fischertechnik interface could be found.");
		}
		
		// drivetrain
		driveTrain = new NormalizedDriveTrainSimpleAlgorithm(
						new NormalizedDriveTrainSimpleFTDemoRobot(iface)
					);
		
		// trusty-robot
		trusty = new SubsumptionTrustyFTDemoRobot(
						new TrustySimpleAlgorithm(
							new TrustySimpleFTDemoRobot(driveTrain)
						),
						iface
					);
		
		// lise
		lise = new SubsumptionLiSe(
						new SubsumptionLiSeFTDemoRobot(
							new LiSeSimpleAlgorithm(
								new LiSeSimpleFTDemoRobot(iface)
							),
							iface
						)
					);
		// because SubsumptionAriadne does'nt know that lise is also subsumption
		((SubsumptionLiSe)lise).getSchedulerPerformer().setDaemon(true);
		((SubsumptionLiSe)lise).getSchedulerPerformer().start();
		lise.search();
		
		// ariadne (non-subsumption)
		ariadne = new AriadneSimpleAlgorithm(
						new AriadneSimpleFTDemoRobot(iface)
					);
		minOrientInterval = 5000; // re-orientation not before these ms are over
	}
// ...
	private static String className =
			"de.jaetzold.art.examples.SubsumptionAriadneFTDemoRobot";
	public static void main(String[] argv) {
		SubsumptionAriadneFTDemoRobot robot =
									new SubsumptionAriadneFTDemoRobot();
		SubsumptionAriadne ariadne = new SubsumptionAriadne(robot);
		ariadne.getSchedulerPerformer().start();	// start subsumption
		
		DriveTrainView driveTrainView =
						new DriveTrainView(robot.driveTrain);
		NormalizedDriveTrainCommands driveTrainCommands =
						new NormalizedDriveTrainCommands(robot.driveTrain);

		TrustyView trustyView = new TrustyView(ariadne);
		TrustyCommands trustyCommands = new TrustyCommands(ariadne);

		LiSeView liseView = new LiSeView(robot.getLiSe());
		LiSeCommands liseCommands = new LiSeCommands(robot.getLiSe());

		AriadneView ariadneView = new AriadneView(robot.getAriadne());
		AriadneCommands ariadneCommands =
							new AriadneCommands(robot.getAriadne());
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());

		java.awt.Panel topPanel = new java.awt.Panel();
		topPanel.setLayout(new java.awt.BorderLayout());
		java.awt.Panel subPanel = new java.awt.Panel();
		subPanel.setLayout(new java.awt.BorderLayout());
		topPanel.add("West", new java.awt.Label("DriveTrain: "));
		topPanel.add("Center", subPanel);
		subPanel.add("Center", driveTrainView);
		subPanel.add("South", driveTrainCommands);
		f.add("North", topPanel);
		
		java.awt.Panel upperMiddlePanel = new java.awt.Panel();
		upperMiddlePanel.setLayout(new java.awt.BorderLayout());
		subPanel = new java.awt.Panel();
		subPanel.setLayout(new java.awt.BorderLayout());
		upperMiddlePanel.add("West", new java.awt.Label("Trusty: "));
		upperMiddlePanel.add("Center", subPanel);
		subPanel.add("Center", trustyView);
		subPanel.add("South", trustyCommands);
		f.add("West", upperMiddlePanel);

		java.awt.Panel lowerMiddlePanel = new java.awt.Panel();
		lowerMiddlePanel.setLayout(new java.awt.BorderLayout());
		subPanel = new java.awt.Panel();
		subPanel.setLayout(new java.awt.BorderLayout());
		lowerMiddlePanel.add("West", new java.awt.Label("Ariadne: "));
		lowerMiddlePanel.add("Center", subPanel);
		subPanel.add("Center", ariadneView);
		subPanel.add("South", ariadneCommands);
		f.add("East", lowerMiddlePanel);

		java.awt.Panel bottomPanel = new java.awt.Panel();
		bottomPanel.setLayout(new java.awt.BorderLayout());
		subPanel = new java.awt.Panel();
		subPanel.setLayout(new java.awt.BorderLayout());
		bottomPanel.add("West", new java.awt.Label("LiSe: "));
		bottomPanel.add("Center", subPanel);
		subPanel.add("Center", liseView);
		subPanel.add("South", liseCommands);
		f.add("South", bottomPanel);

		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
