package de.jaetzold.art.examples;

import de.jaetzold.art.RobotInterfaceFactory;
import de.jaetzold.art.RobotInterface;
import de.jaetzold.util.NotConnectedException;

public class SubsumptionAriadneMSDemoRobot extends SubsumptionAriadneRobot
{
	// only needed for (future) subclasses and test-main
	protected NormalizedDriveTrain driveTrain;
	
	public SubsumptionAriadneMSDemoRobot() {
		super(null, null, 0, null);	// fake parameters, set them afterwards
		
		RobotInterface iface = new RobotInterfaceFactory().getInterface("MS");
		if(iface == null) {
			throw new NotConnectedException("No Mindstorms interface could be found.");
		}
		
		// drivetrain
		driveTrain = new NormalizedDriveTrainSimpleAlgorithm(
						new NormalizedDriveTrainSimpleMSDemoRobot(iface)
					);
		
		// trusty-robot
		trusty = new SubsumptionTrustyMSDemoRobot(
						new TrustySimpleAlgorithm(
							new TrustySimpleMSDemoRobot(driveTrain)
						),
						iface
					);
		
		// lise
		lise = new SubsumptionLiSe(
						new SubsumptionLiSeMSDemoRobot(
							new LiSeSimpleAlgorithm(
								new LiSeSimpleMSDemoRobot(iface)
							),
							iface
						)
					);
		// because SubsumptionAriadne does'nt know that lise is also subsumption
		((SubsumptionLiSe)lise).getSchedulerPerformer().setDaemon(true);
		((SubsumptionLiSe)lise).getSchedulerPerformer().start();
		lise.search();
		
		// ariadne (non-subsumption)
		ariadne = new AriadneSimpleAlgorithm(
						new AriadneSimpleMSDemoRobot(iface)
					);
		minOrientInterval = 5000; // re-orientation not before these ms are over
	}
// ...
	private static String className =
			"de.jaetzold.art.examples.SubsumptionAriadneFTDemoRobot";
	public static void main(String[] argv) {
		SubsumptionAriadneMSDemoRobot robot =
									new SubsumptionAriadneMSDemoRobot();
		SubsumptionAriadne ariadne = new SubsumptionAriadne(robot);
		ariadne.getSchedulerPerformer().start();	// start subsumption
		
		DriveTrainView driveTrainView =
						new DriveTrainView(robot.driveTrain);
		NormalizedDriveTrainCommands driveTrainCommands =
						new NormalizedDriveTrainCommands(robot.driveTrain);

		TrustyView trustyView = new TrustyView(ariadne);
		TrustyCommands trustyCommands = new TrustyCommands(ariadne);

		LiSeView liseView = new LiSeView(robot.getLiSe());
		LiSeCommands liseCommands = new LiSeCommands(robot.getLiSe());

		AriadneView ariadneView = new AriadneView(ariadne);
		AriadneCommands ariadneCommands = new AriadneCommands(ariadne);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		java.awt.Panel names = new java.awt.Panel();
		names.setLayout(new java.awt.GridLayout(4, 1));
		java.awt.Panel ctrlViews = new java.awt.Panel();
		ctrlViews.setLayout(new java.awt.GridLayout(4, 1));
		f.add("West", names);
		f.add("Center", ctrlViews);
		
		names.add(new java.awt.Label("DriveTrain: "));
		java.awt.Panel cvp = new java.awt.Panel();
		java.awt.Panel sub = new java.awt.Panel();
		cvp.add(sub);
		sub.setLayout(new java.awt.BorderLayout());
		sub.add("North", driveTrainView);
		sub.add("South", driveTrainCommands);
		ctrlViews.add(cvp);
		
		names.add(new java.awt.Label("Trusty: "));
		cvp = new java.awt.Panel();
		sub = new java.awt.Panel();
		cvp.add(sub);
		sub.setLayout(new java.awt.BorderLayout());
		sub.add("North", trustyView);
		sub.add("South", trustyCommands);
		ctrlViews.add(cvp);

		names.add(new java.awt.Label("LiSe: "));
		cvp = new java.awt.Panel();
		sub = new java.awt.Panel();
		cvp.add(sub);
		sub.setLayout(new java.awt.BorderLayout());
		sub.add("North", liseView);
		sub.add("South", liseCommands);
		ctrlViews.add(cvp);

		names.add(new java.awt.Label("Ariadne: "));
		cvp = new java.awt.Panel();
		sub = new java.awt.Panel();
		cvp.add(sub);
		sub.setLayout(new java.awt.BorderLayout());
		sub.add("North", ariadneView);
		sub.add("South", ariadneCommands);
		ctrlViews.add(cvp);

		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
// ...
}
