package de.jaetzold.art.examples;

import de.jaetzold.art.BooleanSensor;
import de.jaetzold.art.ConstantSensorPort;
import de.jaetzold.art.subsumption.SchedulerTask;
import de.jaetzold.util.Task;
import de.jaetzold.util.Event;
import de.jaetzold.util.Queue;
import de.jaetzold.util.TaskPerformer;
import de.jaetzold.util.RequeueTask;
import de.jaetzold.util.BeanSupport;	// does some reflect-stuff

import java.beans.PropertyChangeListener;

public class SubsumptionTrusty implements Trusty {
	protected Trusty trusty;
	protected Task move;
	protected Task avoidLeft;
	protected Task avoidRight;
	
	protected SchedulerTask scheduler;
	public SchedulerTask getSchedulerTask() {
		return scheduler;
	}
	protected TaskPerformer schedulerPerformer;
	// to be able to start it and setDaemon
	public TaskPerformer getSchedulerPerformer() {
		return schedulerPerformer;
	}
	
	public SubsumptionTrusty(Robot robot) {
		this.trusty = robot.getTrusty();
		scheduler = new SchedulerTask();
		
// BEGIN SubTrusty:moveBehavior		
		// standard always-active behavior 'move'
		BooleanSensor constantTrue = new BooleanSensor();
		constantTrue.connectWith(new ConstantSensorPort(1));
		move = new Task() {
			public Event perform() {
				trusty.move();
				return null;
			}
		};
		scheduler.addBehaviorFor(new Double(0), constantTrue, move);
// END SubTrusty:moveBehavior		
		
// BEGIN SubTrusty:leftBehavior	
		// behavior for avoiding left side obstacles
		avoidLeft = new Task() {
			public Event perform() {
				trusty.avoidLeft();
				return null;
			}
		};
		scheduler.addBehaviorFor(new Double(1),
								robot.getLeftSensor(),
								avoidLeft);
// END SubTrusty:leftBehavior	
		// behavior for avoiding right side obstacles
		avoidRight = new Task() {
			public Event perform() {
				trusty.avoidRight();
				return null;
			}
		};
		scheduler.addBehaviorFor(new Double(2),
								robot.getRightSensor(),
								avoidRight);
// BEGIN SubTrusty:performer		
		Queue taskQueue = new Queue();
		schedulerPerformer = new TaskPerformer(taskQueue);
		// schedulerPerformer.start(); // start the performer from the outside
		schedulerPerformer.haltPerforming(true);
		taskQueue.post(new RequeueTask(scheduler, taskQueue));
// END SubTrusty:performer		
	}
// BEGIN SubTrusty:robot	
	public interface Robot {
		public Trusty getTrusty();
		public BooleanSensor getLeftSensor();
		public BooleanSensor getRightSensor();
	}
// END SubTrusty:robot	
// BEGIN SubTrusty:trusty	
	public void move() {
		// start algorithm
		if(!schedulerPerformer.isAlive()) {
			// it' either has not been started yet
			try {
				schedulerPerformer.start();
			} catch(IllegalThreadStateException itse) {
				// or already finished execution
			}
		}
		schedulerPerformer.resumePerforming();
	}
	public void stop() {
		// stop algorithm
		schedulerPerformer.haltPerforming();
		try {
			schedulerPerformer.waitForHalt();
		} catch(InterruptedException ie) {
		}
		trusty.stop();
	}
	public void avoidLeft() {
		// TBD: instead of stopping subsumption, the corresponding
		// BooleanSensor could deliver true, to initiate an 'avoid'
		stop();
		trusty.avoidLeft();
	}
// END SubTrusty:trusty	
	public void avoidRight() {
		stop();
		trusty.avoidRight();
	}
// ...	
	public void addPropertyChangeListener(PropertyChangeListener l) {
		BeanSupport.addPropertyChangeListenerFor(trusty, l);
	}
	public void removePropertyChangeListener(PropertyChangeListener l) {
		BeanSupport.removePropertyChangeListenerFor(trusty, l);
	}
	public int getState() {
		return trusty.getState();
	}
	public SubsumptionTrusty() throws Exception {
		this((Robot)Class.forName(
				System.getProperty(
					className +".robotClass",
					"de.jaetzold.art.examples.SubsumptionTrustyRobot"
				)
			).newInstance()
		);
	}
	public static void main(String[] argv) throws Exception {
		int index = 0;
		if(argv.length > index) {
			System.setProperty(className +".robotClass", argv[index++]);
		}
		if(argv.length > index) {
			System.setProperty(argv[index-1] +".trustyClass", argv[index++]);
		}
		SubsumptionTrusty trusty = new SubsumptionTrusty();
		
		TrustyView view = new TrustyView(trusty);
		TrustyCommands commands = new TrustyCommands(trusty);
		
		java.awt.Frame f = new java.awt.Frame(className);
		f.setLayout(new java.awt.BorderLayout());
		f.add("North", view);
		f.add("South", commands);
		
		f.addWindowListener(
			new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent we) {
					System.exit(0);
				}
			}
		);
		f.pack();
		f.show();
	}
	private static String className =
			"de.jaetzold.art.examples.SubsumptionTrusty";
// ...
}
