package de.jaetzold.art.platform;

import de.jaetzold.art.ActuatorPort;
import de.jaetzold.art.Actuator;
import de.jaetzold.art.RobotInterface;
import de.jaetzold.art.Port;

/**	Base class for easy implementation of the interface <code>ActuatorPort</code>.
	The only thing required is to feed the contructor with the desired parameters and implement the factory-method for the peers: <code>getPeerFor(Actuator)</code>
	*/
public abstract class BaseActuatorPort	extends BaseSensorPort
										implements ActuatorPort
{
	public void connectWith(Actuator actuator) {
		super.connectWith(actuator);
		//connectWith((Sensor)actuator);
	}
	public void disconnect(Actuator actuator) {
		super.disconnect(actuator);
		//disconnect((Sensor)actuator);
	}
	
	/**	Default is an empty String.
		*/
	protected String type = "";
	public String getType() {
		return type;
	}
	
	/** Constructs a new ActuatorPort with dumb default values for it's properties. The properties are described in more detail in the description of the interfaces <code>ActuatorPort</code> and  <code>SensorPort</code>
		@see de.jaetzold.art.SensorPort
		@see de.jaetzold.art.ActuatorPort
		*/
	public BaseActuatorPort() {
	}
	
	/** Constructs a new ActuatorPort with the given values for it's properties. The properties are described in more detail in the description of the interfaces <code>ActuatorPort</code> and  <code>SensorPort</code>
		@see de.jaetzold.art.SensorPort
		@see de.jaetzold.art.ActuatorPort
		*/
	public BaseActuatorPort(RobotInterface iface,
							Object identifier,
							int portNumber,
							String type,
							double minimumValue,
							double maximumValue,
							long granularity,
							Port[] excludedPorts) {
		super(iface, identifier, portNumber, minimumValue, maximumValue, granularity, excludedPorts);
		this.type = type;
	}
	
	/**	This is the Factory-method for creating peers. It must be implemented by non-abstract subclasses.
		*/
	public abstract ActuatorPeer getPeerFor(Actuator actuator);
	
/*	protected ActuatorPeer getActuatorPeer(Actuator actuator) {
		ActuatorPeer peer = iface.getActuatorPeer(this, actuator);
		return peer;
	}*/
	
/*	private Object connectMonitor = new Object();
	private boolean connecting = false;
	private boolean disconnecting = false;
	private ActuatorPeer peer;
	public void connectWith(Actuator actuator) {
		synchronized(connectMonitor) {
			if(connecting == true) {
				return;
			}
			connecting = true;
			peer = iface.getActuatorPeer(this, actuator);
			actuator.addPropertyChangeListener(peer);
			actuator.addVetoableChangeListener(peer);
			actuator.setPeer(peer);
			actuator.connectWith(this);
			connecting = false;
		}
	}
	public void disconnect(Actuator actuator) {
		synchronized(connectMonitor) {
			if(disconnecting == true) {
				return;
			}
			disconnecting = true;
			actuator.removePropertyChangeListener(peer);
			actuator.removeVetoableChangeListener(peer);
			actuator.disconnect();
			disconnecting = false;
		}
	}*/
/*	protected int minimalPulseWidth;	// in microseconds
	protected int pulseAccuracy;		// in +-microseconds
	protected int maximumVoltage;		// maximum Suppliable voltage in millivolts
	protected boolean polarityReversable;	// if DC, can the Polarity be changed
	protected boolean isAC;				// AC or DC?
	protected int maximumAutomaticPulses;	// how many different Pulses can be handled automatically
	protected int accuracy;				// in +-millivolts
*/
}
