package de.jaetzold.art.platform;

import de.jaetzold.art.SensorEventDescriptor;
import de.jaetzold.art.SensorEvent;

import java.beans.PropertyChangeListener;

public interface SensorPeer {
	// is that used?
	// public int getIntValue();
	
	/**	Returns the value this peer represents as a double.
		*/
	public double getValue();
	/** Tells whether this SensorPeer delivers discrete values or not. A result of false means that the result of a getValue() can be cast to int without a loss of precision.
		*/
	public boolean getDeliversFloatingPoint();
	/**	Tells whether the connection to the represented sensor is still intact.
		@see de.jaetzold.art.RobotInterface#isConnected()
		*/
	public boolean isConnected();
//	public boolean getBool()
//	public long getCount()
//	public void setCount(long base)
	// AWT uses Event-Masks here, using objects should be more flexible
	/**	Enable delivering the Events described by the given <code>SensorEventDescriptor</code>. Not fully used. Implementors may enable more than the descibed Events an this is pretty much "all" in the current implementations.
		@see de.jaetzold.art.SensorEventDescriptor
		*/
	public void enableEvent(SensorEventDescriptor eventDescriptor);
	/**	Disable delivering the Events described by the given <code>SensorEventDescriptor</code>. Not fully used. Implementors may disable less than the descibed Events an this is pretty much "none" in the current implementations.
		@see de.jaetzold.art.SensorEventDescriptor
		*/
	public void disableEvent(SensorEventDescriptor eventDescriptor);
	
	/**	Generate and deliver a SensorEvent for the current sensors state.
		*/
	public void update();

	/**	Returns the maximum of the expectable error in the value.
		This is just a guess, what means that nothing <em>really</em> important should depend on it. It is meant to give an algorithm using a Sensor connected to this SensorPeer an <em>idea</em> of how precise the value is so that it either can refuse to use it or try to compensate if necessary.
		*/
	public double getPrecision();
	
	/**	The SensorEvents get deliverd to all registered <code>PropertyChangeListener</code>s.
		The PropertyChangeEvent has the SensorEvent as it's new value and the last SensorEvent as the old value. The property name used for this is <code>value</code>.
		This mechanism in the SensorPeer is not really used for anything else right now but may be in future releases.
		@see de.jaetzold.art.Sensor#addPropertyChangeListener(PropertyChangeListener)
		@see #processEvent(SensorEvent)
		*/	
	public void addPropertyChangeListener(PropertyChangeListener l);
	/** Remove the specified listener.
		@see #addPropertyChangeListener(PropertyChangeListener)
		*/
	public void removePropertyChangeListener(PropertyChangeListener l);
	/**	Deliver the given <code>SensorEvent</code> to all Listeners. Maybe update internal cache of the value.
		@see #addPropertyChangeListener(PropertyChangeListener)
		*/
	public void processEvent(SensorEvent event);
//	later ...
//	public void setPollingInterval(int microseconds);
/* In AWT this is not in the Peer. What is the reason?
	public void addSensorListener(SensorListener listener);
	public void removeSensorListener(SensorListener listener);
*/
}
