package de.jaetzold.art.platform;

import de.jaetzold.art.AngleSensor;

/**	A Peer representing a servo motor.
	<p>
	A Servo is a device which has control over the absolute position of an axle. It is calibrated so that it knows what one turn of the axle means and where a specific angle is. It is possible (and for a usual servo motor very likely) that not all angles are reachable.
	*/
public interface ServoPeer extends StepperMotorPeer {
	/**	The AngleSensor for measuring the position.
		*/
	public AngleSensor getAngleSensor();
	/**	Set's the AngleSensor to use for measuring the position. This may involve a substancial decrease in speed and precision and is therefore not a good idea for now. It is more for a future use.
		*/
	public void setAngleSensor(AngleSensor sensor);
	/**	Give the servo a chance to calibrate. This is meant as a hint that it would do no harm if it takes some time now to calibrate if necessary. If the Peer can't decide by itself whether calibration is necessary this should also be interpreted as a hint that the precision is getting worse.
		*/
	public void calibrate();
}
