package de.jaetzold.art.platform;

import de.jaetzold.art.CountSensor;

// Note: it is crucial for the current implementation of StepperMotor that
// its Peer is also a MotorPeer

/**	This is an interface for peers representing a stepper motor.
	<p>
	The position is represented through a separate <code>CountSensor</code> because a StepperMotor does behave also like a simple Motor and therefore <code>getValue()</code> already has a different meaning.
	<p>
	Because a StepperMotor has control over the amount of movement it can also move at a certain speed. The speed is different to power because speed means how far in which time, no matter how hard it is ...
	*/
public interface StepperMotorPeer extends MotorPeer {
	/**	Move the stepper motor the given amount of steps.
		*/
	public void step(double value);
	/**	Returns the Sensor descriping the position of this StepperMotor.
		*/
	public CountSensor getCountSensor();
	/**	Set's the CountSensor to use for counting the steps. This may involve a substancial decrease in speed and precision and is therefore not a good idea for now. It is more for a future use.
		*/
	public void setCountSensor(CountSensor sensor);
	/**	The current speed at which the Motor is moving.
		*/
	public double getSpeed();
	/**	Set the speed at which the Motor should move.
		*/
	public void setSpeed(double speed);
}
