package de.jaetzold.art.platform.lego;

import de.jaetzold.art.platform.PacketFilter;
import de.jaetzold.art.platform.CorruptPacketException;

import java.util.Hashtable;
import java.util.Enumeration;

import de.jaetzold.util.Event;
import de.jaetzold.util.NoImmediateExecutionException;

public class RCXPacketFilterTable	implements	PacketFilter
{
	private static boolean debug = true;
	// This class depends on the not-documented behaviour of java.lang.Byte
	// to be equal to any other Byte, which wraps the same primitive
	// But to try to avoid unfindable bugs do a simple test:
	static {
		if(! new Byte((byte)1).equals(new Byte((byte)1))) {
			throw new IncompatibleClassChangeError("Implementation of java.lang.Byte is not as expected.");
		}
	}

	public Event process(byte[] packet)
		throws CorruptPacketException
	{
		// look whether there is a filter registered for the ID of the given packet
		PacketFilter filter = getFilterFor(packet[0]);
		if(filter == null) { return null; }	// no filter available
		return filter.process(packet);
	}
	
	public boolean done() {
		return false;
	}

	private Hashtable filterTable = new Hashtable();

	public PacketFilter getFilterFor(byte id) {
		return (PacketFilter)filterTable.get(new Byte(id));
	}

	public synchronized void removeFilterFor(byte id) {
		if(filterTable.remove(new Byte(id)) != null) {
			// because some filter has been removed
			this.notifyAll();
		}
	}

	public synchronized void removeAll(PacketFilter filter) {
		// this maybe could be easier achieved by calling
		// Vector tmp = new Vector();
		// tmp.add(filter);
		// filterTable.values().removeAll(tmp);
		// but that would definitely depend on 1.2
		Enumeration e = filterTable.keys();
		while(e.hasMoreElements()) {
			Object key = e.nextElement();
			if(filter.equals(filterTable.get(key))) {
				filterTable.remove(key);
				this.notifyAll();
			}
		}
	}

	private int defaultSetFilterRetry = 100;
	public int getDefaultSetFilterRetry() {
		return defaultSetFilterRetry;
	}
	public void setDefaultSetFilterRetry(int defaultSetFilterRetry) {
		this.defaultSetFilterRetry = defaultSetFilterRetry;
	}

	public synchronized void setFilterFor(byte id, PacketFilter filter) {
		setFilterFor(id, filter, defaultSetFilterRetry);
	}
	public synchronized void setFilterFor(byte id, PacketFilter filter, int retry) {
		try {
			setFilterFor(id, filter, true, retry);
		} catch(NoImmediateExecutionException niee) {
			// will never arrive ... but nevertheless:
			if(debug) {
				System.err.println(getClass().getName()
									+": should never happen:");
				niee.printStackTrace();
			}
		}
	}
	public synchronized void setFilterFor(	byte id,
											PacketFilter filter,
											boolean wait,
											int retry)
			throws NoImmediateExecutionException
	{
		PacketFilter old = getFilterFor(id);
		// if there is no filter for that id, or the old filter accepts to be replaced
		// just insert the new one
		if(old == null || old.done()) {
			filterTable.put(new Byte(id), filter);
		} else {
			if(!wait) {
				throw new NoImmediateExecutionException("Previous filter not finished yet.");
			}

			// wait for the old filter to be removed or finished
			do {
				try {
					this.wait(retry);
				} catch(InterruptedException ie) {
					// clear interrupt-status
					Thread.currentThread().interrupted();
				}
				old = getFilterFor(id);
			} while(old != null && !old.done());

			// now it is safe to insert the new filter
			filterTable.put(new Byte(id), filter);
		}
	}
}
