package de.jaetzold.art.platform.lego;

import java.io.FilterOutputStream;
import java.io.OutputStream;
import java.io.IOException;

import de.jaetzold.util.StringObservable;	// just for debug

public class RCXPacketOutputStream extends FilterOutputStream {	
	protected byte[] header;
	public byte[] getHeader() {
		return header;
	}
	public void setHeader(byte[] header) {
		this.header = header;
	}

	public RCXPacketOutputStream(OutputStream target, byte[] header) {
		super(target);
		setHeader(header);
	}

	protected int defaultWriteTimeout = 0;	// 0 or less means disabled
	public int getDefaultWriteTimeout() {
		return defaultWriteTimeout;
	}
	public void setDefaultWriteTimeout(int defaultWriteTimeout) {
		this.defaultWriteTimeout = defaultWriteTimeout;
	}

	public void writePacket(byte[] content)
			throws IOException
	{
		writePacket(content, 0, content.length, defaultWriteTimeout);
	}

	public void writePacket(byte[] content, int timeout)
			throws IOException
	{
		writePacket(content, 0, content.length, timeout);
	}

	public void writePacket(byte[] content, int off, int len, int timeout)
			throws IOException
	{
		byte[] packet = new byte[len*2+2+header.length];

		// copy contents of header at the beginning of the packet
		System.arraycopy(header, 0, packet, 0, header.length);

		// content[off], ~content[off], ..., content[off+len-1], ~content[off+len-1],
		// sumOf(content[off], ..., content[off+len-1]), ~sumOf(...)
		byte checksum = (byte)0;
		for(int ci=off, pi=header.length;
						ci<off+len;
						ci++, pi+=2) {
			packet[pi] = content[ci];
			packet[pi+1] = (byte) ~ content[ci];
			checksum += content[ci];
		}

		packet[packet.length-2] = checksum;
		packet[packet.length-1] = (byte) ~ checksum;
		
		if(debug) {
			System.out.println(getClass().getName() +": "
								+"writing "
								+StringObservable.toHexString(packet));
		}
		out.write(packet, 0, packet.length);

	}
	
	private static boolean debug;
	static {
		String v = null;
		v = System.getProperty("de.jaetzold.art.RCXPacketOutputStream"+".debug");
		if(v==null) {
			v = System.getProperty("de.jaetzold.art"+".debug");
		}
		if(v==null) {
			v = System.getProperty("debug");
		}
		// results to false if v!="true" (ignoring case)
		debug = Boolean.valueOf(v).booleanValue();
	}
}
