package de.jaetzold.util;

import java.awt.*;
import java.awt.event.*;

/** Modaler Dialog mit einem TextField fuer eine Nachricht, sowie optional bis zu 3 Knoepfen. Diese Klasse kann verwendet werden um einfach Nachrichten an den Benutzer der Oberflaeche zu geben, bzw. Rueckfragen zu starten. Der Rueckgabewert von run() gibt Aufschluss darueber welcher Knopf gedrueckt wurde.
*/
public class AlertDialog extends Dialog {
    /**	Konstante fuer keinen gedrueckten Knopf. */
    public static final int NONE = -1;
    /**	Konstante fuer den OK Knopf. */
    public static final int OK = 1;
    /**	Konstante fuer den CANCEL Knopf. */
    public static final int CANCEL = 2;
    /**	Konstante fuer den IGNORE Knopf. */
    public static final int IGNORE = 3;

    /** Erzeugt einen neuen Dialog mit den angegebenen Parametern. Ist der String fuer den Text auf einem Knopf leer wird ein Defaultwert genommen. Ist der String == null wird der Knopf nicht angezeigt.
        Die Knoepfe die angezeigt werde werden in der Reihenfolge ok, ignore und cancel angezeigt. 
        @param message die darzustellende Nachricht fuer das TextField.
        @param ok Text der auf dem OK Knopf stehen soll.
        @param cancel Text der auf dem CANCEL Knopf stehen soll.
        @param ignore Text der auf dem IGNORE Knopf stehen soll.
        @param mainFrame der Frame dessen Input in der Modalschleife abgefangen werden soll.
        */
    public AlertDialog(String message, String ok, String cancel, String ignore, Frame mainFrame) {
        super(mainFrame, "Attention!", true);
        setLayout(new BorderLayout());
        
        Panel buttons = new Panel();
        buttons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        add(buttons, BorderLayout.SOUTH);

        TextArea messageArea = new TextArea(message, 4, 40, TextArea.SCROLLBARS_NONE);
        messageArea.setEditable(false);
        add(messageArea, BorderLayout.CENTER);

        Button b;
        if(ok != null) {
            b = new Button(ok.equals("") ? "OK" : ok);
            b.addActionListener(new ActionListener()
                                {
                public void actionPerformed(ActionEvent ae) {
                    klicked(OK);
                }
                                });
            buttons.add(b);
            b.requestFocus();
        }
        if(ignore != null) {
            b = new Button(ignore.equals("") ? "IGNORE" : ignore);
            b.addActionListener(new ActionListener()
                                {
                public void actionPerformed(ActionEvent ae) {
                    klicked(IGNORE);
                }
                                });
            buttons.add(b);
            b.requestFocus();
        }
        if(cancel != null) {
            b = new Button(cancel.equals("") ? "CANCEL" : cancel);
            b.addActionListener(new ActionListener()
                                {
                public void actionPerformed(ActionEvent ae) {
                    klicked(CANCEL);
                }
                                });
            buttons.add(b);
            b.requestFocus();
        }
    }

    /**	Wenn geklickt wurde wird diesem Monitor notifyAll() geshickt. */
    protected Object buttonKlickMonitor = new Object();
    /** Die Nummer des gedreuckten Knopfes. Diese entspricht den in der Klasse definierten Konstanten. */
    protected int klickedButtonId = NONE;
    /** Stellt den AlertDialog dar und kommt erst zurueck wenn auf einen der Knoepfe geklickt wurde. */
    public int run() {
        pack();
        show();
// brauch man bei modalem dialog nicht, da "show" bereis blockiert ...
/*        synchronized(buttonKlickMonitor) {
            try {
                buttonKlickMonitor.wait();
            } catch(InterruptedException ie) {
                
            }
        }*/

        return klickedButtonId;
    }

    /** Wird von den ActionListenern der Knoepfe aufgerufen wenn geklickt wurde.
        @param buttonId die zu dem entsprechenden Knopf gehoerige Konstante.
        */
    public void klicked(int buttonId) {
        klickedButtonId = buttonId;
        setVisible(false);
        
        synchronized(buttonKlickMonitor) {
            buttonKlickMonitor.notifyAll();
        }
    }
}
