package de.jaetzold.util;

import java.beans.PropertyVetoException;
import java.beans.PropertyChangeEvent;

public class BinaryOperatorStateDecider implements StateDecider {
	private boolean cacheable;
	public boolean isCacheable() {
		return cacheable;
	}
	public double nextStateValue(double lastState, double newState) {
		double result = 0;
		if(operator != null) {
			result = operator.combine(lastState, newState);
		}
		return result;
	}
	
	protected BinaryOperator operator;
	public BinaryOperator getBinaryOperator() {
		return operator;
	}
	public void setBinaryOperator(BinaryOperator operator)
			throws PropertyVetoException
	{
		if(this.operator != null) {
			throw new PropertyVetoException(
						"An operator can not be set again once it had been set to a non-null value.",
						new PropertyChangeEvent(this,
												"binaryOperator",
												this.operator,
												operator));
		}
		this.operator = operator;
	}
	
	public BinaryOperatorStateDecider() {
		this(null);
	}
	
	public BinaryOperatorStateDecider(BinaryOperator operator) {
		this(operator, false);
	}
	
	public BinaryOperatorStateDecider(	BinaryOperator operator,
										boolean cacheable) {
		try {
			setBinaryOperator(operator);
		} catch(PropertyVetoException pve) {
		}
		this.cacheable = cacheable;
	}
}
