package de.jaetzold.util;

public class ByteArray {
	private static final Debug debug = new Debug("de.jaetzold.util.ByteArray");
	
	// necessary for using byte[] as key in Map
	private byte[] buffer;
	private int hash;
	public ByteArray(byte[] bytes) {
		buffer = bytes;
		if(bytes != null) {
			for(int i=0; i<bytes.length; i++) {
				hash += bytes[i] + 128;	// is this a good hash-function?
			}
		}
	}
	public boolean equals(Object o) {
		if(o != null && o.getClass() == getClass()
				&& java.util.Arrays.equals(buffer, ((ByteArray)o).buffer)) {
			return true;
		}
		return false;
	}
	public int hashCode() {
		return hash;
	}
	
	public String toString() {
		return getClass().getName()
				+"[" +StringObservable.toHexString(buffer) +"]";
	}
    /**	Macht eine komplette Kopie des uebergebenen Arrays. */
	public static byte[] copy(byte[] source) {
        return copy(source, 0, source.length);
	}
	
    /**	Liefert eine Kopie von dem spezifizierten Subarray des uebergebenen Arrays.
    @param from das Array welches kopiert werden soll.
    @param offset der index in from ab dem kopiert werden soll.
    @param length die Anzahl Elemente die kopiert werden soll.
    @exception ArrayIndexOutOfBoundsException if the specified region is not completely in from.
    	*/
    public static byte[] copy(byte[] from, int offset, int length) {
        byte[] result = new byte[length];
        System.arraycopy(from, offset, result, 0, length);
        return result;
    }

		/** Increase the size of buffer if full, or create it if null. After a call to this method one can be sure, that the result
			<ul>
			<li>is non-null</li>
			<li>contains everything what buffer contains, at the same position</li>
			<li>has a length greater than filled</li>
			<li>is == buffer, if and only if buffer meets the above requirements.</li>
			</ul>
			@param increase a suggestion for the increase in size.
		*/
	public static byte[] manageBufferSize(byte[] buffer,
											int filled,
											int increase) {
		byte[] result = buffer;
		if(buffer == null) {
			result = new byte[Math.max(1, increase)];
		} else if(filled >= buffer.length) {
			result = new byte[	Math.max(buffer.length, filled)
								+Math.max(buffer.length, increase)];
			System.arraycopy(buffer, 0, result, 0, buffer.length);	

			if(debug.debug) {
				debug.printInfo("increased buffer to "
						+result.length
						+": " +StringObservable
								.toHexString(result,
											0,
											buffer.length,
											" ")
						);
			}
		}
		return result;
	} 

    public static short toShort(byte higher, byte lower) {
        int result = 0;
        //int h = sign*higher; // im grunde dasselbe wie bei lower ...
        int l = (lower<0 ? lower+256 : lower);
        result = (higher*256 + l);
        if(debug.debug) {
            byte[] tmp = new byte[2];
            byte[] tmp2 = toBytes((short)result);
            tmp[0] = higher; tmp[1] = lower;
			byte tmp3 = tmp2[0];
            tmp2[0] = tmp2[1]; tmp2[1] = tmp3;
            debug.printInfo("toShort: "
						+StringObservable.toHexString(tmp)
                        +" = " + (short)result
                        +" = " + StringObservable.toHexString(tmp2));
}
        return (short) result;
    }
    public static byte[] toBytes(short value) { // Lower byte first (at position 0)
        byte[] result = new byte[2];
        int sign = (value<0 ? -1 : 1);
        // lower byte:
        int l = value%256;
        result[0] = (byte)(l>128 ? l-256 : l);
        // higher byte
        result[1] = (byte)(value >> 8);

        return result;
    }
}
