package de.jaetzold.util;

public class CombinedRange implements Range {
	private final Debug debug = new Debug("de.jaetzold.util.CombinedRange");
	{ if(debug.debug) debug.objectIdentifier = debug.getIdentifierFor(this); }
	
	private boolean isCacheableCache = false;
	public boolean isCacheable() {
		if(!isCacheableCache) {
			if(ranges!=null) {
				for(int i=0; i<ranges.length; i++) {
					if(ranges[i] == null
							|| !ranges[i].isCacheable()) {
						return false;
					}
				}
				isCacheableCache = true;
			}
		}
		return isCacheableCache;
	}
	
	private Range[] ranges;
	
	public CombinedRange(Range theOne, Range theOther) {
		this(new Range[] {theOne, theOther});
	}
	
	public CombinedRange(Range[] ranges) {
		this.ranges = new Range[ranges.length];
		System.arraycopy(ranges, 0, this.ranges, 0, ranges.length);
	}
	
	public boolean contains(double value) {
		for(int i=0; i<ranges.length; i++) {
			// any of the ranges has to be non-null and contain the value
			if(ranges[i] != null && ranges[i].contains(value)) {
				if(debug.debug) {
					debug.printInfo(ranges[i] +" contained " +value);
				}
				return true;
			}
		}
		if(debug.debug) {
			debug.printInfo("no Range in " +ranges +" contained " +value);
		}
		return false;
	}
}
