package de.jaetzold.util;

// all Functions are some kind of Map, because they could be represented by a table (the Set of all values is finite)
public interface Conversion {
	/** Tells whether this instance will in the future produce the same result for the same arguments.
		This can be false first and then true at a later time, but it is not legal to return false here after a return of true in the past, because the true answer before was wrong then.
		@return true if this instance guarantees, that any further call to convert(double) with the same argument returns the same result
	*/
	public boolean isCacheable();
	/** Tells whether this instance really delivers floating point values. That means whether the result of a convert() can not always be cast to long without a loss of precision.
		*/
	public boolean getDeliversFloatingPoint();
	public double convert(double value);
}
