package de.jaetzold.util;

import java.beans.PropertyVetoException;
import java.beans.PropertyChangeEvent;

public abstract class LinkedStateDecider implements StateDecider {		
	private boolean isCacheableCache = false;
	public boolean isCacheable() {
		if(!isCacheableCache) {
			if(first!=null && second != null) {
				isCacheableCache = first.isCacheable() && second.isCacheable();
			}
		}
		return isCacheableCache;
	}
	
	protected StateDecider first;
	protected StateDecider second;

	public StateDecider getFirstDecider() {
		return first;
	}
	public StateDecider getSecondDecider() {
		return second;
	}

	public synchronized void setFirstDecider(StateDecider first)
			throws PropertyVetoException
	{
		if(this.first != null) {
			throw new PropertyVetoException(
						"A decider can not be set again once it had been set to a non-null value.",
						new PropertyChangeEvent(this,
												"firstDecider",
												this.first,
												first));
		}
		this.first = first;
	}
	public synchronized void setSecondDecider(StateDecider second)
			throws PropertyVetoException
	{
		if(this.second != null) {
			throw new PropertyVetoException(
						"A decider can not be set again once it had been set to a non-null value.",
						new PropertyChangeEvent(this,
												"secondDecider",
												this.second,
												second));
		}
		this.second = second;
	}
	
	public LinkedStateDecider() {
		// don't initialize with deciders!
	}
	public LinkedStateDecider(StateDecider first, StateDecider second) {
		try {
			setFirstDecider(first);
			setSecondDecider(second);
		} catch(PropertyVetoException pve) {
			// At least in this implementation this point will be never reached
		}
	}
}
