package de.jaetzold.util;

/**	A interface for classes that can check whether a specific primitive value is contained in some thing (typical a mathematical interval) or not.
	As opposed to the interface java.util.Set a Range is not required to be able to deliver a Set of Objects for each value that is considered to be in it.
	@see java.util.Set
	*/	
public interface Range {
	/** Tells whether this instance will in the future produce the same result for the same arguments.
		This can be false first and then true at a later time, but it is not legal to return false here after a return of true in the past, because the true answer before was wrong then.
		@return true if this instance guarantees, that any further call to contains() with the same arguments returns the same result
	*/
	public boolean isCacheable();
	/** Tells whether the given value is in the Range or not.
		@return true, if the given value could be found inside the Range, false otherwise
		*/	
	public boolean contains(double value);
}
