package de.jaetzold.util;

import java.awt.*;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.comm.SerialPort;
import javax.comm.CommPortIdentifier;
import javax.comm.PortInUseException;
import javax.comm.UnsupportedCommOperationException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
// eigenes weil "setChanged()" protected ist
import de.jaetzold.util.Observable;
import java.util.Observer;

public class SerialPortOptionsPanel extends Panel {
    protected CommPortIdentifier[] portArray;
    protected Choice portC;
    protected Hashtable identifiersOfPortsOpened;

    protected SerialPort serialPort;
    protected Choice databitsC;
    protected Choice stopbitsC;
    protected Choice parityC;
    protected Choice flowcontrolC;
    protected TextField baudrateTF;

    protected Observable observable;

    protected static String appName;
    public static void setAppname(String _appName) {
        appName = _appName;
    }
    public static String getAppname() {
        return appName;
    }

    protected int timeout = 2000;
    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }
    public int getTimeout() {
        return timeout;
    }
    
    public SerialPortOptionsPanel()
      {
        Enumeration enum = CommPortIdentifier.getPortIdentifiers();
        Vector vec = new Vector();
        while(enum.hasMoreElements()) {
            CommPortIdentifier cpi = (CommPortIdentifier) enum.nextElement();
            if(cpi.getPortType() == cpi.PORT_SERIAL)
                vec.addElement(cpi);
        }
        portArray = new CommPortIdentifier[vec.size()];
        vec.copyInto(portArray);

        this.createPanelWithChoiceFor(portArray);
    }

    public SerialPortOptionsPanel(CommPortIdentifier[] portArray)
      {
        this.createPanelWithChoiceFor(portArray);
      }

    private void createPanelWithChoiceFor(CommPortIdentifier[] portArray)
     {
        if(portArray.length < 1)
            throw new IllegalArgumentException("No Ports in Array");
        
        setLayout(new BorderLayout());

        Panel p = new Panel();
        p.setLayout(new GridLayout(1, 2));
        add(p,"Center");

        Panel titles = new Panel();
        p.add(titles);
        titles.setLayout(new GridLayout(6, 1));
        Panel options = new Panel();
        p.add(options);
        options.setLayout(new GridLayout(6, 1));

        this.portArray = portArray;
        portC = new Choice();
        for(int i=0; i<portArray.length; i++) {
            portC.add(portArray[i].getName());
        }
        titles.add(new Label("port:", Label.RIGHT));
        options.add(portC);

        identifiersOfPortsOpened = new Hashtable();
// irgendwie darauf achten was mit dem bisherigen port passiert ...
        portC.addItemListener(new ItemListener()
                              {
            public void itemStateChanged(ItemEvent ie) {
                Choice sourceChoice = (Choice)ie.getSource();
                int index = sourceChoice.getSelectedIndex();
                // der CommPortIdentifier zum selektierten namen
                CommPortIdentifier identifier =
                    SerialPortOptionsPanel.this.portArray[index];
                // nur was machen falls selektion auch wirklich neu
                if(identifier.getName().equals(serialPort.getName())) {
                    return;
                } 
                SerialPort port = null;

                // um den port einstellen zu koennen muss man ihn oeffnen
                // falls nicht schon geschehen
                if(!identifiersOfPortsOpened.containsKey(identifier)){
                try {
                    port = (SerialPort)identifier.open(appName, timeout);
                        identifiersOfPortsOpened.put(identifier, port);
                    } catch(PortInUseException e) {
                        e.printStackTrace();
                        sourceChoice.select(serialPort.getName());
                        return;
                    }
                } else {
                    port = (SerialPort)identifiersOfPortsOpened.get(identifier);
                }
                // gewaehlten Port setzen um optionen einzutragen
                // falls das nicht klappt wird eh auf den alten zurueckgesetzt
                try {
                    setSerialPort(port);
                } catch(UnsupportedCommOperationException ucoe) {
                    ucoe.printStackTrace();
                    sourceChoice.select(serialPort.getName());
                }
            }
                              });

        this.createCommonViews(titles, options);
        int index = 0;
        CommPortIdentifier identifier = portArray[index];
        while(this.serialPort==null) {
            identifier = portArray[index];
            try {
                this.serialPort = (SerialPort)identifier.open(appName, timeout);
            } catch(PortInUseException piue) {
                piue.printStackTrace();
                index++;
                if(index >= portArray.length) {
                    for(int i=0; i<portArray.length; i++) {
                        System.err.println(portArray[i].getName());
                        System.err.flush();
                    }
                    throw new IllegalArgumentException("No Port in Array is available");
                }
            }
        }
        identifiersOfPortsOpened.put(identifier, this.serialPort);
        try {
            setSerialPort(this.serialPort); // um parameter einzutragen;
        } catch(UnsupportedCommOperationException ucoe) {
            ucoe.printStackTrace();
            // offenbar sind die defaultparameter des ports schon zuviel ...
            throw new RuntimeException("Das sollte nicht passieren ...");
        }
        portC.select(index);
    }

    // UnsupportedCommOperationException abfangen und Dialog geben
    // Methoden schreiben um Parameter des serialPort zu aendern
    public SerialPortOptionsPanel(SerialPort serialPort)
        // Falls z.B. die Oberflaeche bestimmte bereits eingestellte Parametwr nicht unterstuetzt
    				throws UnsupportedCommOperationException
      {
          setLayout(new BorderLayout());

          Panel p = new Panel();
          p.setLayout(new GridLayout(1, 2));
          add(p,"Center");

          Panel titles = new Panel();
          p.add(titles);
          titles.setLayout(new GridLayout(5, 1));
          Panel options = new Panel();
          p.add(options);
          options.setLayout(new GridLayout(5, 1));

          this.createCommonViews(titles, options);

          this.setSerialPort(serialPort);
      }

    protected void createCommonViews(Panel titles, Panel options)
        // Falls z.B. die Oberflaeche bestimmte bereits eingestellte Parametwr nicht unterstuetzt
      {
        Label l = new Label("Baudrate: ", Label.RIGHT);
        titles.add(l);
        baudrateTF = new TextField();
        options.add(baudrateTF);

        l = new Label("Databits: ", Label.RIGHT);
        titles.add(l);
        databitsC = new Choice();
        // Achtung, Reihenfolge ist fuer setSerialPortParams() relevant
        // Loesung: Hashtables machen die index auf Zustandsobjekt mappen.
        databitsC.add("5");
        databitsC.add("6");
        databitsC.add("7");
        databitsC.add("8");
        options.add(databitsC);

        l = new Label("Stopbits: ", Label.RIGHT);
        titles.add(l);
        stopbitsC = new Choice();
        // Achtung, Reihenfolge ist fuer setSerialPortParams() relevant
        stopbitsC.add("1");
        stopbitsC.add("2");
        stopbitsC.add("1.5");
        options.add(stopbitsC);

        l = new Label("Parity: ", Label.RIGHT);
        titles.add(l);
        parityC = new Choice();
        // Achtung, Reihenfolge ist fuer setSerialPortParams() relevant
       	parityC.add("None");
        parityC.add("Odd");
        parityC.add("Even");
        parityC.add("Mark");
        parityC.add("Space");
        options.add(parityC);

        l = new Label("Flowcontrol: ", Label.RIGHT);
        titles.add(l);
        flowcontrolC = new Choice();
        // Achtung, Reihenfolge ist fuer setFlowControlMode() relevant
        flowcontrolC.add("None");
        flowcontrolC.add("RTS/CTS");
        flowcontrolC.add("XON/XOFF");
        options.add(flowcontrolC);

        Panel p = new Panel();
        add(p, "South");
        p.setLayout(new BorderLayout());

        Checkbox cb = new Checkbox("Als Starteinstellung nehmen");

        p.add(cb, "East");
        p.add(p = new Panel(), "East");

        Button b = new Button("OK");
        b.addActionListener(new ActionListener()
                            {
            public void actionPerformed(ActionEvent ae) {
// auch noch nicht ganz sicher ob nicht doch ein port reingeschmuggelt werden kann und die oberflaeche "merkt" es nicht ...
                try {
                    setSerialPortParams();
                    setFlowControlMode();
                } catch(UnsupportedCommOperationException ucoe) {
                    ucoe.printStackTrace();
                    try{
                        setSerialPort(serialPort);
                    } catch(UnsupportedCommOperationException ucoe2) {
                        System.err.println("Sollte eigentlich nie passieren!");System.err.flush();
                        ucoe2.printStackTrace();
                    }
                    return;
                }
                
                int index = portC.getSelectedIndex();
                SerialPort portToBeClosed;
                Enumeration e = identifiersOfPortsOpened.elements();
                while(e.hasMoreElements()) {
                    portToBeClosed = (SerialPort)e.nextElement();
                    if(portToBeClosed != serialPort) {
                        portToBeClosed.close();
                    }
                }
                // siehe "cancel-button"
                // identifiersOfPortsOpened.clear();
                notifyObservers(serialPort);
                closeUpperDialogIfAny();
            }
                            });
        p.add(b);

        b = new Button("Cancel");
        b.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                // custom fuer test: close auf alle ports ...
                Enumeration e = identifiersOfPortsOpened.elements();
                while(e.hasMoreElements()) {
                    SerialPort port = (SerialPort) e.nextElement();
                    port.close();
                }
                closeUpperDialogIfAny();
            }
        });
        p.add(b);

        this.observable = new Observable();
      }
    
    protected boolean closeUpperDialogIfAny() {
        Dialog d;

        Container c = getParent();
        while(!(c instanceof Dialog) && c != null) {
            c = c.getParent();
        }
        if(c == null) {
            return false;
        }

        d = (Dialog) c;
        d.setVisible(false);
        return true;
    }

    public synchronized void addObserver(Observer o) {
        observable.addObserver(o);
    }
    
    public synchronized void deleteObserver(Observer o) {
        observable.deleteObserver(o);
    }

    public void notifyObservers(){
        observable.notifyObservers();
    }
    
    public void notifyObservers(Object arg) {
        observable.notifyObservers(arg);
    }
    
    public void setSerialPort(SerialPort serialPort)
        // Falls z.B. die Oberflaeche bestimmte bereits eingestellte Parametwr nicht unterstuetzt
    				throws UnsupportedCommOperationException
      {
          SerialPort oldPort = this.serialPort;

          this.serialPort = serialPort;

          try {
              setFlowControlMode(serialPort.getFlowControlMode());
              setSerialPortParams(serialPort.getBaudRate(),
                                  serialPort.getDataBits(),
                                  serialPort.getStopBits(),
                                  serialPort.getParity());
          } catch(UnsupportedCommOperationException ucoe) {
              this.serialPort = oldPort;
              throw ucoe;
          }

          observable.setChanged();
      }

    public void setFlowControlMode()
        throws UnsupportedCommOperationException
      {
          int mode = 0;

          switch(flowcontrolC.getSelectedIndex()) {
              case 0:
                  mode = SerialPort.FLOWCONTROL_NONE;
                  break;
              case 1:
                  mode = SerialPort.FLOWCONTROL_RTSCTS_IN
                  | SerialPort.FLOWCONTROL_RTSCTS_OUT;
                  break;
              case 2:
                  mode = SerialPort.FLOWCONTROL_XONXOFF_IN
                  | SerialPort.FLOWCONTROL_XONXOFF_OUT;
                  break;
          }

          setFlowControlMode(mode);
      }

    public void setFlowControlMode(int mode)
        throws UnsupportedCommOperationException
      {
          // setzen der Choice auf der Oberflaeche
          // es werden nur die auch auf der Oberflaeche
          // einstellbaren modes nicht abgewiesen
          int oldFlowcontrol = flowcontrolC.getSelectedIndex();

          if(mode	== SerialPort.FLOWCONTROL_NONE) {
              flowcontrolC.select(0);
          } else if(mode == (SerialPort.FLOWCONTROL_RTSCTS_IN
                             | SerialPort.FLOWCONTROL_RTSCTS_OUT)) {
              flowcontrolC.select(1);
          } else if(mode == (SerialPort.FLOWCONTROL_XONXOFF_IN
                             | SerialPort.FLOWCONTROL_XONXOFF_OUT)) {
              flowcontrolC.select(2);
          } else {
              throw new UnsupportedCommOperationException(
                                                          "FlowControlMode "
                                                          +mode
                                                          +" not Supported by "
                                                          +getClass().getName());
          }
          try {
              serialPort.setFlowControlMode(mode);
          } catch(UnsupportedCommOperationException ucoe) {
              // damit Oberflaeche und serialPort keine verschiedenen Einstellungen haben
              flowcontrolC.select(oldFlowcontrol);
              throw ucoe;
          }
      }

    public void setSerialPortParams()
    				throws UnsupportedCommOperationException
      {
          int baudrate = Integer.parseInt(baudrateTF.getText());
          int databits = databitsC.getSelectedIndex();
          int stopbits = stopbitsC.getSelectedIndex();
          int parity = parityC.getSelectedIndex();

          switch(databits) {
              case 0:
                  databits = SerialPort.DATABITS_5;
                  break;
              case 1:
                  databits = SerialPort.DATABITS_6;
                  break;
              case 2:
                  databits = SerialPort.DATABITS_7;
                  break;
              case 3:
                  databits = SerialPort.DATABITS_8;
                  break;
          }

          switch(stopbits) {
              case 0:
                  stopbits = SerialPort.STOPBITS_1;
                  break;
              case 1:
                  stopbits = SerialPort.STOPBITS_2;
                  break;
              case 2:
                  stopbits = SerialPort.STOPBITS_1_5;
                  break;
          }

          switch(parity) {
              case 0:
                  parity = SerialPort.PARITY_NONE;
                  break;
              case 1:
                  parity = SerialPort.PARITY_ODD;
                  break;
              case 2:
                  parity = SerialPort.PARITY_EVEN;
                  break;
              case 3:
                  parity = SerialPort.PARITY_MARK;
                  break;
              case 4:
                  parity = SerialPort.PARITY_SPACE;
                  break;
          }

          setSerialPortParams(baudrate, databits, stopbits, parity);
      }

    public void setSerialPortParams(int baudrate,
                                    int databits,
                                    int stopbits,
                                    int parity)
    				throws UnsupportedCommOperationException
      {

          String oldBaudrate = baudrateTF.getText();
          int oldDatabits = databitsC.getSelectedIndex();
          int oldStopbits = stopbitsC.getSelectedIndex();
          int oldParity = parityC.getSelectedIndex();

          baudrateTF.setText("" +baudrate);

          try {
              switch(databits) {
                  case SerialPort.DATABITS_5:
                      databitsC.select(0);
                      break;
                  case SerialPort.DATABITS_6:
                      databitsC.select(1);
                      break;
                  case SerialPort.DATABITS_7:
                      databitsC.select(2);
                      break;
                  case SerialPort.DATABITS_8:
                      databitsC.select(3);
                      break;
                  default:
                      throw new UnsupportedCommOperationException(
                                                                  "Databits Mode "
                                                                  +databits
                                                                  +" not Supported by "
                                                                  +getClass().getName());
              }

              switch(stopbits) {
                  case SerialPort.STOPBITS_1:
                      stopbitsC.select(0);
                      break;
                  case SerialPort.STOPBITS_2:
                      stopbitsC.select(1);
                      break;
                  case SerialPort.STOPBITS_1_5:
                      stopbitsC.select(2);
                      break;
                  default:
                      throw new UnsupportedCommOperationException(
                                                                  "Stopbits Mode "
                                                                  +stopbits
                                                                  +" not Supported by "
                                                                  +getClass().getName());
              }

              switch(parity) {
                  case SerialPort.PARITY_NONE:
                      parityC.select(0);
                      break;
                  case SerialPort.PARITY_ODD:
                      parityC.select(1);
                      break;
                  case SerialPort.PARITY_EVEN:
                      parityC.select(2);
                      break;
                  case SerialPort.PARITY_MARK:
                      parityC.select(3);
                      break;
                  case SerialPort.PARITY_SPACE:
                      parityC.select(4);
                      break;
                  default:
                      throw new UnsupportedCommOperationException(
                                                                  "Parity Mode "
                                                                  +parity
                                                                  +" not Supported by "
                                                                  +getClass().getName());
              }

              serialPort.setSerialPortParams(baudrate, databits, stopbits, parity);

          } catch(UnsupportedCommOperationException ucoe) {
              // damit Oberflaeche und serialPort keine verschiedenen Einstellungen haben
              baudrateTF.setText(oldBaudrate);
              databitsC.select(oldDatabits);
              stopbitsC.select(oldStopbits);
              parityC.select(oldParity);
              throw ucoe;
          }
      }
}
