package de.jaetzold.util;

/**	A class defining an intervall for numbers.
	A Range can be checked whether it contains a specific value or not.
	*/	
public class SingleRange implements Range {
	/** Stores the upper border of this range.
	*/
	private double upperBorder;
	/** Stores the lower border of this range.
	*/
	private double lowerBorder;
	
	/** Tells whether values that are equal to the upper border are considered to be out of the range. 
	*/
	private boolean openUpperBorder;
	/** Tells whether values that are equal to the lower border are considered to be out of the range. 
	*/
	private boolean openLowerBorder;
	
	public SingleRange() {
		this(1, 1);
	}
	public SingleRange(double threshold) {
		this(threshold, Double.POSITIVE_INFINITY);
	}
	public SingleRange(double lower, double upper) {
		this(lower, upper, false, false);
	}
	public SingleRange(	double lower,
						double upper,
						boolean openLowerBorder,
						boolean openUpperBorder) {
		if(lower>upper) {
			throw new IllegalArgumentException("Lower border greater than upper border. Use class ComplementRange or CombinationRange and You can get what You probably wanted here.");
		} else {
			lowerBorder = lower;
			upperBorder = upper;
		}
		openUpperBorder = openUpperBorder;
		openLowerBorder = openLowerBorder;
	}
	
	/** Tells whether the given value is in the Range or not.
		@return true, if the given value could be found inside the Range, false otherwise
		*/	
	public boolean contains(double value) {
		if(value>lowerBorder && value<upperBorder) {
			return true;
		}
		if(!openLowerBorder && value==lowerBorder) {
			return true;
		}
		if(!openUpperBorder && value==upperBorder) {
			return true;
		}
		return false;
	}
	

	public boolean isCacheable() {
		return true;
	}
}
