package de.jaetzold.util;

// maps a combination of primitive values (except boolean) to int
// a Map does that already, but for Objects
// a StateDecider should be much faster
// if only primitives are needed

// changed to deliver double because of problems with subclasses of statesensor
/**	This interface can be used as an abstraction of a State-Change-Matrix of a finite state machine. But as it is dynamic it is much more flexible.
	*/
public interface StateDecider {
	/** Tells whether this instance will in the future produce the same state for the same value argument, independent of the actualState.
		This can be false first and then true at a later time, but it is not legal to return false here after a return of true in the past, because the true answer before was wrong then.
		@return true if this instance guarantees, that any further call to nextStateValue() with the same value-argument returns the same state
	*/
	public boolean isCacheable();
	/**	Return the next state. The result may depend on the given actualState and the current value but is allowed to depend on anything else. Be sure to have <code>isCacheable()</code> return a correct result then.
		@see #isCacheable() 
		*/
	public double nextStateValue(double actualState, double value);
	/* maybe
	public int nextStateValue(int actualState, long value);
	public int nextStateValue(long[] values);
	public int nextStateValue(double[] values);
	public Object nextState(Object value);
	*/
}
