package de.jaetzold.util;

import java.beans.PropertyVetoException;
import java.beans.PropertyChangeEvent;

public class StateDeciderLinkedStateDecider extends LinkedStateDecider {	
	public double nextStateValue(double actualState, double value) {
		// needs no sync/copy to be Thread-safe
		// because first and second cannot be set to null once there had
		// been assigned a non-null value to them
		double firstValue = 0;
		double secondValue = 0;
		
		if(linker != null) {
			if(first != null) {
				firstValue = first.nextStateValue(actualState, value);
			}
			if(second != null) {
				secondValue = second.nextStateValue(actualState, value);
			}
			return linker.nextStateValue(firstValue, secondValue);
		} else {
			return 0;
		}
	}
	
	private boolean isCacheableCache = false;
	public boolean isCacheable() {
		if(!isCacheableCache) {
			if(linker != null && second != null) {
				// the first is not important
				isCacheableCache = linker.isCacheable() && second.isCacheable();
			}
		}
		return isCacheableCache;
	}
	
	protected StateDecider linker;

	public StateDecider getLinkDecider() {
		return linker;
	}

	public synchronized void setLinkDecider(StateDecider linker)
			throws PropertyVetoException
	{
		if(this.linker != null) {
			throw new PropertyVetoException(
						"A decider can not be set again once it had been set to a non-null value.",
						new PropertyChangeEvent(this,
												"linkerDecider",
												this.linker,
												linker));
		}
		this.linker = linker;
	}

	public StateDeciderLinkedStateDecider() {
		// don't initialize with deciders!
	}
	public StateDeciderLinkedStateDecider(StateDecider first,
										StateDecider second,
										StateDecider linker) {
		super(first, second);
		try {
			setLinkDecider(linker);
		} catch(PropertyVetoException pve) {
		}
	}
}
