package de.jaetzold.util;

public class StateLinkedStateDecider extends LinkedStateDecider {
	public boolean isCacheable() {
		// this one is only dependent of the second decider
		if(second != null) {
			return second.isCacheable();
		} else {
			return false;
		}
	}
		
	public double nextStateValue(double actualState, double value) {
		// needs no sync/copy to be Thread-safe
		// because first and second cannot be set to null once there had
		// been assigned a non-null value to them
		
		if(second != null) {
			if(first != null) {
				return second.nextStateValue(first.nextStateValue(actualState,
																value),
											value);
			} else {
				return second.nextStateValue(0, value);
			}
		} else {
			return 0;
		}
	}

	public StateLinkedStateDecider() {
		// don't initialize with deciders!
	}
	public StateLinkedStateDecider(StateDecider first, StateDecider second) {
		super(first, second);
	}
}
