package de.jaetzold.util;

import java.beans.PropertyVetoException;
import java.beans.PropertyChangeEvent;

public class TransitionMatrixStateDecider implements StateDecider {	
	public double nextStateValue(double actualState, double value) {
		// needs no sync/copy to be Thread-safe
		// because matrix is not visible to the outside
		// anyone who wants to access matrix via getDeciders() gets a copy.
		// if this method gets called while matrix is being initialized
		// it should do no harm too
		// (if System.arracopy() doesn't set any arrayelement first to
		//  a non-null value and then to null)
		
		if(matrix!=null) {
			if(actualState>=0 && actualState<matrix.length
					&& matrix[(int)actualState] != null) {
				return matrix[(int)actualState]
							.nextStateValue(actualState, value);
			}
		}
		return 0;
	}
	
	public boolean isCacheable() {
		return false;	// because the decider is selected by the actual state
	}
	
	private StateDecider[] matrix;
	
	public StateDecider getDeciders(int index) {
		return matrix[index];
	}
	public StateDecider[] getDeciders() {
		StateDecider[] result = null;
		if(matrix != null) {
			result = new StateDecider[matrix.length];
			System.arraycopy(matrix, 0, result, 0, matrix.length);
		}
		return result;
	}
	public synchronized void setDeciders(int index, StateDecider decider)
			throws PropertyVetoException
	{
		if(this.matrix[index] == null) {
			this.matrix[index] = decider;
		} else {
			throw new PropertyVetoException(
						"A Decider in the matrix can not be set again after there had been assigned a non-null value to it.",
						new PropertyChangeEvent(this,
												"deciders",
												this.matrix,
												matrix));
		}
	}
	public synchronized void setDeciders(StateDecider[] matrix)
			throws PropertyVetoException
	{
		if(this.matrix == null) {
			this.matrix = new StateDecider[matrix.length];
		// if System.arracopy() sets any arrayelement first to a
		// non-null value and then to null, nextState() has to be sync'ed too
			System.arraycopy(matrix, 0, this.matrix, 0, matrix.length);
		} else {
			throw new PropertyVetoException(
						"The Matrix can not be set again after there had been assigned a non-null value.",
						new PropertyChangeEvent(this,
												"deciders",
												this.matrix,
												matrix));
		}
	}

	public TransitionMatrixStateDecider() {
		// don't initialize with deciders!
	}
	public TransitionMatrixStateDecider(StateDecider[] matrix) {
		try {
			setDeciders(matrix);
		} catch(PropertyVetoException pve) {
			// At least in this implementation this point will be never reached
		}
	}
	public TransitionMatrixStateDecider(int[][] matrix) {
		StateDecider[] deciders = new StateDecider[matrix.length];
		for(int i=0; i<matrix.length; i++) {
			deciders[i] = new TransitionTableStateDecider(matrix[i]);
		}
		try {
			setDeciders(deciders);
		} catch(PropertyVetoException pve) {
			// At least in this implementation this point will be never reached
		}
	}
}
