package de.jaetzold.util;

import java.beans.PropertyVetoException;
import java.beans.PropertyChangeEvent;

// its the corresponding class for RangeStateDecider, but this one
// decides only based on the actualState
// what about negative values?

public class TransitionTableStateDecider implements StateDecider {
	public double nextStateValue(double actualState, double value) {
		if(actualState<0 || actualState>=transitionTable.length) {
			return 0;
		}
		return transitionTable[(int)actualState];
	}
	
	// transitionTable can never be modified after it has been set.
	// that way the behaviour is predictable
	// but which transition is used depends on the actual state
	// and therefor this decider is not cacheable
	public boolean isCacheable() {
		return false;
	}

	private int[] transitionTable;
	private boolean gotTransitionTableSet = false;
	public int[] getTransitionTable() {
		int[] result = null;
		if(transitionTable != null) {
			result = new int[transitionTable.length];
			System.arraycopy(	transitionTable, 0,
								result, 0,
								transitionTable.length);
		}
		return result;
	}
	public synchronized void setTransitionTable(int[] transitionTable)
			throws PropertyVetoException
	{
		if(!gotTransitionTableSet) {
			if(transitionTable != null) {
				this.transitionTable = new int[transitionTable.length];
				System.arraycopy(	transitionTable, 0,
									this.transitionTable, 0,
									transitionTable.length);
			} else {
				this.transitionTable = null;
			}
			gotTransitionTableSet = true;
		} else {
			throw new PropertyVetoException(
						"TransitionTable can be set only once.",
						new PropertyChangeEvent(this,
												"transitionTable",
												this.transitionTable,
												transitionTable));
		}
	}
	
	public TransitionTableStateDecider() {
	}
	public TransitionTableStateDecider(int[] transitionTable) {
		try {
			setTransitionTable(transitionTable);
		} catch(PropertyVetoException pve) {
			// At least in this implementation this point will be never reached
		}
	}
}
