package de.jaetzold.util.apps;

import de.jaetzold.util.*;

import java.awt.Frame;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Observer;
import java.util.TooManyListenersException;
import javax.comm.CommPort;
import javax.comm.SerialPort;
import javax.comm.SerialPortEventListener;
import javax.comm.SerialPortEvent;

/* CommController */
public class CommController implements CommandLineListener, Observer {


    public static void main(String[] argv) {
        new CommController().runAWT(new Object());
    }

    protected OutputStream outStream;
    protected CommPort commPort;

    protected int inputBufferSize = 256;
    /**	Hauptschleife fuer das Lesen vom CommPort. Die gelesenen Bytes werden in Hex-Strings umgewandelt (ohne 0x davor).
        */
    public void processCommInput(InputStream inStream) {
        // hier wird davon ausgegangen, dass der Aufrufer sich
        // um synchronisierung des Zugriffs auf den Stream kuemmert
        // die Ueberpruefungen ob auch wirklich was da ist
        // muessten eigentlich unnoetig sein.
        int available;
        try {
            if((available = inStream.available()) <= 0) {
                        output.append("** nothing available! **\n");
                        return;
                    }
        } catch(IOException ioe) {
            ioe.printStackTrace();
            return;
        }

        byte[] buf = new byte[inputBufferSize];
        int len = 0;
        // Schleife damit falls Inputbuffer zu klein nochmal gelesen wird
        while(available > 0) {
            try {
                len = inStream.read(buf);
            } catch(IOException ioe) {
                ioe.printStackTrace();
                output.append("** " +ioe +" **\n");
                return;
            }

            if(len == -1) {
                output.append("** inputStream is closed! **\n");
                return;
            }
            // vor der Ausgabe die bytes noch in hex-String verwandeln ...
            StringBuffer hexString = new StringBuffer();
            for(int i=0; i<len; i++) {
                int tmp = (int)buf[i];
                if(tmp < 0) {
                    tmp+=256;
                }
                hexString.append(Integer.toHexString(tmp));
                if((hexString.length()-i)%2 != 0) {
                    hexString.insert(i*3, '0');
                }
                hexString.append(' ');
            }
            output.append(hexString);
            available-=len;
        }
    }

    /** Schreibt den uebergebenen String auf den CommPort. Der String wird dazu vorher als eine Folge von Hex-Zahlen mit 2 Ziffern interpretiert und in entsprechende byte-Werte umgewandelt. Ist die Anzahl der Zeichen ungerade wird das letzte Zeichen einfach unterschlagen.
        */
    public void processCommandLine(String line) {
        StringTokenizer tok = new StringTokenizer(line);
        String sub = "";
        while(tok.hasMoreTokens()) {
            sub = sub + tok.nextToken();
        }
        byte[] bytesToSend = new byte[sub.length()/2];
        if(sub.length()%2 != 0) {
            output.append("** odd number of characters in input. **\n");
       }
        
        int index = 0;
        while(sub.length() >= index*2 + 2) {
            bytesToSend[index] = (byte)Short.parseShort(sub.substring(index*2,
                                                              index*2 +2),
                                                16);
                        index++;
        }
        
        if(outStream == null) {
            output.append("** no outputStream available! **\n");
            return;
        }

        // echo:
        output.append("\n" +output.getPrompt() + line.trim() +"\n");
        // zum schreiben exklusiven Zugriff auf den stream verlangen
        // dann kann der zwischendurch auch immer mal neu gesetzt werden
        synchronized(outStream) {
            try {
                outStream.write(bytesToSend);
            } catch(IOException ioe) {
                ioe.printStackTrace();
                output.append("** " +ioe +" **\n");
            }
        }
    }

    public void update(java.util.Observable panel, Object port) {
        CommPort newPort;
        try {
            newPort = (CommPort)port;
        } catch(ClassCastException cce) {
            cce.printStackTrace();
            terminate(port);
            return;
        }
        OutputStream newOutputStream = null;
        try {
            newOutputStream = newPort.getOutputStream();
        } catch(IOException ioe) {
            ioe.printStackTrace();
        }
        // alten nur schliessen falls neuer commPort
        if(!newPort.equals(commPort) && outStream != null) {
            synchronized(outStream) {
                try {
                    outStream.close();
                } catch(IOException ioe) {
                    ioe.printStackTrace();
                }
                outStream = null;
            }
        }
        if(newOutputStream != null)
            outStream = newOutputStream;

        if(!newPort.equals(commPort)) {
            if(commPort!= null)
                commPort.close();
            commPort = newPort;
            if(commPort instanceof SerialPort) {
                try {
                    ((SerialPort)commPort).addEventListener(new SerialPortEventListener()
                                                            {
                        public void serialEvent(SerialPortEvent se) {
                            if(se.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
                                InputStream tmp = null;
                                try {
                                    tmp = ((CommPort)se.getSource()).getInputStream();
                                } catch(IOException ioe) {
                                    ioe.printStackTrace();
                                }
                                if(tmp == null)
                                    terminate("Data available but no inputStream.");
                                processCommInput(tmp);
                            }
                        }
                                                            });
                    ((SerialPort)commPort).notifyOnDataAvailable(true);
					// quick hack to test Cybermaster:
System.out.println("" +((SerialPort)commPort).isDTR() +((SerialPort)commPort).isRTS());
                    ((SerialPort)commPort).setDTR(true);
                    //((SerialPort)commPort).setDTR(false);
                    ((SerialPort)commPort).setRTS(false);
                } catch(TooManyListenersException tmle) {
                    tmle.printStackTrace();
                }
            }
        }
    }

    protected TextOutput output;
    private boolean startedAWT = false;
    public synchronized void runAWT(Object sender) {
        if(startedAWT) {
            return;
        }
        String appName = (getClass().getName()
                          + " : "
                          + System.getProperty("os.name", ""));

        Frame mainFrame = new Frame(appName);
        mainFrame.setLayout(new BorderLayout());
        mainFrame.addWindowListener(new WindowAdapter()
                                    {
            public void windowClosing(WindowEvent we) {
                terminate("App closed through closebutton");
            }
                                    });

        output = new TextOutput(10, 30);
        output.setPrompt("-> ");
        mainFrame.add("Center", output);
        
        CommandLine input = output;
        input.addCommandLineListener(this);

        SerialPortOptionsPanel.setAppname(appName);
        SerialPortOptionsPanel options = new SerialPortOptionsPanel();
        options.addObserver(this);
        Panel p = new Panel(new BorderLayout());
        p.add("East", options);
        mainFrame.add("North", p);

        startedAWT = true;
        mainFrame.pack();
        mainFrame.show();
    }

    public void terminate(Object sender) {
        // close ports ...
        System.err.println(sender); System.err.flush();
        try {
            synchronized(outStream) {
                try {
                    outStream.close();
                } catch(IOException ioe) {
                    ioe.printStackTrace();
                }
                if(commPort!= null)
                    commPort.close();
            }
        } catch(NullPointerException npe) {
            System.err.println("out: " +outStream);
        }
        System.exit(0);
    }
}
